#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2014 Martin Raspaud

# Author(s):

#   Martin Raspaud <martin.raspaud@smhi.se>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""For Panu
"""

from logging.handlers import TimedRotatingFileHandler
import re
import time
import os

KEYS = {"%Y": r"\d{4}",
        "%y": r"\d{2}",
        "%m": r"\d{2}",
        "%d": r"\d{2}",
        "%H": r"\d{2}",
        "%M": r"\d{2}",
        "%S": r"\d{2}",
        "%j": r"\d{3}"}

class PanusTimedRotatingFileHandler(TimedRotatingFileHandler):
    """Like TimedRotatingFileHandler with a custom filename template.
    """

    def __init__(self, template, *args, **kwargs):

        self.template = template
        current_time = int(time.time())
        if kwargs.get("utc", False):
            time_tuple = time.gmtime(current_time)
        else:
            time_tuple = time.localtime(current_time)
        filename = time.strftime(self.template, time_tuple)

        self.match = os.path.basename(self.template)
        for key, val in KEYS.iteritems():
            self.match = self.match.replace(key, val)

        self.match = re.compile(self.match)
        TimedRotatingFileHandler.__init__(self, filename, *args, **kwargs)

    def getFilesToDelete(self):
        """
        Determine the files to delete when rolling over.

        More specific than the earlier method, which assumed the date to be a
        suffix.
        """
        dirname, basename = os.path.split(self.baseFilename)
        del basename
        filenames = os.listdir(dirname)
        result = []
        for filename in filenames:
            if self.match.match(filename):
                result.append(os.path.join(dirname, filename))
        result.sort(reverse=True)
        if len(result) < self.backupCount:
            result = []
        else:
            result = result[:len(result) - self.backupCount]
        return result

    def doRollover(self):
        """
        do a rollover; If there is a backup count, then we have to get a list of
        matching filenames, sort them and remove the oldest ones.
        """
        if self.stream:
            self.stream.close()
            self.stream = None
        # get the time that this sequence started at and make it a TimeTuple
        current_time = int(time.time())
        now = time.localtime(current_time)[-1]
        if self.utc:
            time_tuple = time.gmtime(current_time)
        else:
            time_tuple = time.localtime(current_time)
        self.baseFilename = time.strftime(self.template, time_tuple)
        if self.backupCount > 0:
            for old_file in self.getFilesToDelete():
                os.remove(old_file)
        self.stream = self._open()
        new_rollover_at = self.computeRollover(current_time)
        while new_rollover_at <= current_time:
            new_rollover_at = new_rollover_at + self.interval
        # If DST changes and midnight or weekly rollover, adjust for this.

        if (self.when == 'MIDNIGHT' or self.when.startswith('W')) and \
                not self.utc:
            dst_at_rollover = time.localtime(new_rollover_at)[-1]
            if now != dst_at_rollover:
                if not now:  # DST kicks in before next rollover,
                             # so we need to deduct an hour
                    addend = -3600
                else:           # DST bows out before next rollover,
                                # so we need to add an hour
                    addend = 3600
                new_rollover_at += addend
        self.rolloverAt = new_rollover_at





# -*- coding: utf-8 -*-
#
# Copyright (c) 2014, 2015
#
# Author(s):
#
#   Panu Lahtinen <panu.lahtinen@fmi.fi>
#   Martin Raspaud <martin.raspaud@smhi.se>
#
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

'''Helper functions for Trollduction
'''

import numpy as np
import os
import xml_read
from mpop.projector import get_area_def
from pyresample.geometry import Boundary
import logging
from ConfigParser import ConfigParser

LOGGER = logging.getLogger(__name__)


def read_config_file(fname, config_item=None):
    '''Read config file to dictionary.
    '''

    endswith = os.path.splitext(fname)[1]
    if endswith == ".xml":
        return xml_read.parse_xml(xml_read.get_root(fname))
    elif endswith in ['.ini', '.cfg']:
        config = ConfigParser()
        config.read(fname)
        conf_dict = dict(config.items(config_item))
        conf_dict["config_file"] = fname
        conf_dict["config_item"] = config_item
        return conf_dict
    else:
        raise NotImplementedError("Can only parse xml and .ini config files"
                                  "for now")


def get_maximum_extent(area_def_names):
    '''Get maximum extend needed to produce all defined areas.
    '''
    maximum_area_extent = [None, None, None, None]
    for area in area_def_names:
        extent = get_area_def(area)

        if maximum_area_extent[0] is None:
            maximum_area_extent = list(extent.area_extent)
        else:
            if maximum_area_extent[0] > extent.area_extent[0]:
                maximum_area_extent[0] = extent.area_extent[0]
            if maximum_area_extent[1] > extent.area_extent[1]:
                maximum_area_extent[1] = extent.area_extent[1]
            if maximum_area_extent[2] < extent.area_extent[2]:
                maximum_area_extent[2] = extent.area_extent[2]
            if maximum_area_extent[3] < extent.area_extent[3]:
                maximum_area_extent[3] = extent.area_extent[3]

    return maximum_area_extent


def get_maximum_extent_ll(area_def_names):
    '''Get maximum extend needed to produce all the defined areas
    given in *area_def_names*.
    '''
    maximum_area_extent = [None, None, None, None]

    for area in area_def_names:
        area_def = get_area_def(area['definition'])

        lons, lats = get_area_boundaries(area_def)
        left_lon, down_lat, right_lon, up_lat = \
            np.min(lons.side4), \
            np.min(lats.side3), \
            np.max(lons.side2), \
            np.max(lats.side1)

        if maximum_area_extent[0] is None:
            maximum_area_extent = [left_lon, down_lat, right_lon, up_lat]
        else:
            if maximum_area_extent[0] > left_lon:
                maximum_area_extent[0] = left_lon
            if maximum_area_extent[1] > down_lat:
                maximum_area_extent[1] = down_lat
            if maximum_area_extent[2] < right_lon:
                maximum_area_extent[2] = right_lon
            if maximum_area_extent[3] < up_lat:
                maximum_area_extent[3] = up_lat

    return maximum_area_extent


def get_maximum_ll_borders(area_def_names):
    '''Get maximum extend needed to produce all the defined areas
    given in *area_def_names*.
    '''
    maximum_area_extent = [None, None, None, None]

    for area in area_def_names:
        area_def = get_area_def(area['definition'])

        lons, lats = get_area_boundaries(area_def)
        left_lon, down_lat, right_lon, up_lat = \
            np.min(lons.side4), \
            np.min(lats.side3), \
            np.max(lons.side2), \
            np.max(lats.side1)

        if maximum_area_extent[0] is None:
            maximum_area_extent = [left_lon, down_lat, right_lon, up_lat]
        else:
            if maximum_area_extent[0] > left_lon:
                maximum_area_extent[0] = left_lon
            if maximum_area_extent[1] > down_lat:
                maximum_area_extent[1] = down_lat
            if maximum_area_extent[2] < right_lon:
                maximum_area_extent[2] = right_lon
            if maximum_area_extent[3] < up_lat:
                maximum_area_extent[3] = up_lat

    return maximum_area_extent


def get_area_boundaries(area_def):
    '''Get area boundaries from area definition.
    '''

    # upper boundary
    lonlat = np.array([area_def.get_lonlat(0, i)
                       for i in range(area_def.x_size)])
    up_lons = lonlat[:, 0]
    up_lats = lonlat[:, 1]

    # lower boundary
    lonlat = np.array([area_def.get_lonlat(area_def.y_size - 1, i)
                       for i in range(area_def.x_size)])
    down_lons = lonlat[:, 0]
    down_lats = lonlat[:, 1]

    # left boundary
    lonlat = np.array([area_def.get_lonlat(i, 0)
                       for i in range(area_def.y_size)])
    left_lons = lonlat[:, 0]
    left_lats = lonlat[:, 1]

    # right boundary
    lonlat = np.array([area_def.get_lonlat(i, area_def.x_size - 1)
                       for i in range(area_def.y_size)])
    right_lons = lonlat[:, 0]
    right_lats = lonlat[:, 1]

    return (Boundary(up_lons, right_lons, down_lons, left_lons),
            Boundary(up_lats, right_lats, down_lats, left_lats))


def get_indices_from_boundaries(boundary_lons, boundary_lats,
                                lons, lats, radius_of_influence):
    """Find relevant indices from grid boundaries using the 
    winding number theorem"""

    valid_index = _get_valid_index(boundary_lons.side1, boundary_lons.side2,
                                   boundary_lons.side3, boundary_lons.side4,
                                   boundary_lats.side1, boundary_lats.side2,
                                   boundary_lats.side3, boundary_lats.side4,
                                   lons, lats, radius_of_influence)

    return valid_index


def get_angle_sum(lons_side1, lons_side2, lons_side3, lons_side4):
    '''Calculate angle sum for winding number theorem.  Note that all
    the sides need to be connected, that is:

    lons_side[-1] == lons_side2[0], 
    ...
    lons_side4[-1] == lons_side1[0]
    '''
    angle_sum = 0
    for side in (lons_side1, lons_side2, lons_side3, lons_side4):
        side_diff = np.sum(np.diff(side))
        idxs, = np.where(np.abs(side_diff) > 180)
        if idxs:
            side_diff[idxs] = (np.abs(side_diff[idxs]) - 360) * \
                np.sign(side_diff[idxs])
        angle_sum += np.sum(side_diff)

    return angle_sum


def _get_valid_index(lons_side1, lons_side2, lons_side3, lons_side4,
                     lats_side1, lats_side2, lats_side3, lats_side4,
                     lons, lats, radius_of_influence):
    """Find relevant indices from grid boundaries using the 
    winding number theorem"""

    earth_radius = 6370997.0

    # Coarse reduction of data based on extrema analysis of the boundary
    # lon lat values of the target grid
    illegal_lons = (((lons_side1 < -180) | (lons_side1 > 180)).any() or
                    ((lons_side2 < -180) | (lons_side2 > 180)).any() or
                    ((lons_side3 < -180) | (lons_side3 > 180)).any() or
                    ((lons_side4 < -180) | (lons_side4 > 180)).any())

    illegal_lats = (((lats_side1 < -90) | (lats_side1 > 90)).any() or
                    ((lats_side2 < -90) | (lats_side2 > 90)).any() or
                    ((lats_side3 < -90) | (lats_side3 > 90)).any() or
                    ((lats_side4 < -90) | (lats_side4 > 90)).any())

    if illegal_lons or illegal_lats:
        # Grid boundaries are not safe to operate on
        return np.ones(lons.size, dtype=np.bool)

    # Find sum angle sum of grid boundary
    angle_sum = get_angle_sum(lons_side1, lons_side2,
                              lons_side3[::-1], lons_side4[::-1])

    # Buffer min and max lon and lat of interest with radius of interest
    lat_min = min(lats_side1.min(), lats_side2.min(),
                  lats_side3.min(), lats_side4.min())
    lat_min_buffered = lat_min - np.degrees(float(radius_of_influence) /
                                            earth_radius)
    lat_max = max(lats_side1.max(), lats_side2.max(), lats_side3.max(),
                  lats_side4.max())
    lat_max_buffered = lat_max + np.degrees(float(radius_of_influence) /
                                            earth_radius)

    max_angle_s2 = max(abs(lats_side2.max()), abs(lats_side2.min()))
    max_angle_s4 = max(abs(lats_side4.max()), abs(lats_side4.min()))
    lon_min_buffered = lons_side4.min() - \
        np.degrees(float(radius_of_influence) /
                   (np.sin(np.radians(max_angle_s4)) * earth_radius))

    lon_max_buffered = lons_side2.max() + \
        np.degrees(float(radius_of_influence) /
                   (np.sin(np.radians(max_angle_s2)) * earth_radius))

    # From the winding number theorem follows:
    # angle_sum possiblilities:
    # -360: area covers north pole
    #  360: area covers south pole
    #    0: area covers no poles
    # else: area covers both poles
    if round(angle_sum) == -360:
        LOGGER.debug("Area covers north pole")
        # Covers NP
        valid_index = (lats >= lat_min_buffered)
    elif round(angle_sum) == 360:
        LOGGER.debug("Area covers south pole")
        # Covers SP
        valid_index = (lats <= lat_max_buffered)
    elif round(angle_sum) == 0:
        LOGGER.debug("Area covers no poles")
        # Covers no poles
        valid_lats = (lats >= lat_min_buffered) * (lats <= lat_max_buffered)

        if lons_side2.min() > lons_side4.max():
            # No date line crossing
            valid_lons = (lons >= lon_min_buffered) * \
                (lons <= lon_max_buffered)
        else:
            # Date line crossing
            seg1 = (lons >= lon_min_buffered) * (lons <= 180)
            seg2 = (lons <= lon_max_buffered) * (lons >= -180)
            valid_lons = seg1 + seg2

        valid_index = valid_lats * valid_lons
    else:
        LOGGER.debug("Area covers both poles")
        # Covers both poles, don't reduce
        return True
        # valid_index = np.ones(lons.size, dtype=np.bool)

    return valid_index


def overlapping_timeinterval(start_end_times, timelist):
    """From a list of start and end times check if the current time interval
    overlaps with one or more"""

    starttime, endtime = start_end_times
    for tstart, tend in timelist:
        if ((tstart <= starttime and tend > starttime) or
                (tstart < endtime and tend >= endtime)):
            return tstart, tend
        elif (tstart >= starttime and tend <= endtime):
            return tstart, tend

    return False



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2014 Martin Raspaud

# Author(s):

#   Martin Raspaud <martin.raspaud@smhi.se>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Test websockets
"""

# doesn't work

# import socket, threading, time

# def handle(s):
#     print repr(s.recv(4096))
#     s.send('''
# HTTP/1.1 101 Web Socket Protocol Handshake\r
# Upgrade: WebSocket\r
# Connection: Upgrade\r
# WebSocket-Origin: http://localhost:8888\r
# WebSocket-Location: ws://localhost:9876/\r
# WebSocket-Protocol: sample
#   '''.strip() + '\r\n\r\n')
#     time.sleep(1)
#     s.send('\x00hello\xff')
#     time.sleep(1)
#     s.send('\x00world\xff')
#     s.close()

# s = socket.socket()
# s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
# s.bind(('', 9876))
# s.listen(1)
# while True:
#     t, _ = s.accept()
#     threading.Thread(target=handle, args=(t,)).start()

import cherrypy
from ws4py.server.cherrypyserver import WebSocketPlugin, WebSocketTool
from ws4py.websocket import WebSocket
import threading
import time

cherrypy.config.update({'server.socket_port': 9000})
WebSocketPlugin(cherrypy.engine).subscribe()
cherrypy.tools.websocket = WebSocketTool()

idx = """
<!DOCTYPE html>
<html xmlns="http://www.w3.org/1999/xhtml">
  <head>
    <title>Web Socket Example</title>
    <meta charset="UTF-8">
    <script>
      window.onload = function() {
        var s = new WebSocket("ws://localhost:9876/");
        s.onopen = function(e) { alert("opened"); }
        s.onclose = function(e) { alert("closed"); }
        s.onmessage = function(evt) { $('#msg').append('<p>'+evt.data+'</p>');
                }; 
      };
    </script>
  </head>
    <body>
      <div id="holder" style="width:600px; height:300px"></div>
    </body>
</html>
""".strip()

idx2 = """
<!DOCTYPE html>
<html>
<head>
<script src='http://ajax.googleapis.com/ajax/libs/jquery/1.4.2/jquery.min.js'></script>
<script>
    $(document).ready(function() {
            function debug(str) {
                $('#debug').append('<p>'+str+'</p>');
                };
            ws = new WebSocket('ws://localhost:9000/ws');

/* Define websocket handlers */
      ws.onmessage = function(evt) {
                $('#msg').append(evt.data+'<br />');
                document.scrollTop = document.scrollHeight;
                };
      ws.onclose = function() {
                debug('socket closed');
                };
      ws.onopen = function() {
                debug('connected...');
                ws.send('Hello.\\n');
            };
            });
/* Define click handler */

    $(document).click(function () {
                ws.send('Click!');
            });
</script>
</head>
<body>
    <div id='debug'></div>
    <div id='msg'></div>
</body>
</html>
""".strip()

class Root(object):
    @cherrypy.expose
    def index(self):
        #return 'some HTML with a websocket javascript connection'
        return idx2

    @cherrypy.expose
    @cherrypy.tools.websocket(on=False)
    def ws(self):
        # you can access the class instance through
        handler = cherrypy.request.ws_handler
        #return handler

item = Root()

from ws4py.messaging import TextMessage
from posttroll.subscriber import Subscribe

#cherrypy.config.update({'/ws': {'tools.websocket.on': True,
#                                'tools.websocket.handler_cls': WebSocket}})
cherrypy.tree.mount(Root(), "/", config={'/ws': {'tools.websocket.on': True,
                                'tools.websocket.handler_cls': WebSocket}})

#cherrypy.quickstart(item, '/', config={'/ws': {'tools.websocket.on': True,
#                                               'tools.websocket.handler_cls': EchoWebSocket}})
#pause
cherrypy.engine.start()

def send(msg):
    cherrypy.engine.publish("websocket-broadcast", TextMessage(msg))

try:
    with Subscribe(services=[""], addr_listener=True) as sub:
        for msg in sub.recv(1):
            if msg:
                if msg.type in ["log.debug", "log.info",
                                "log.warning", "log.error",
                                "log.critical"]:
                    send(msg.type + " " + msg.subject + " " +
                         msg.sender + " " +
                         str(msg.data) + " " +
                         str(msg.time))

                elif msg.binary:
                    send(msg.subject + " " +
                         msg.sender + " " +
                         msg.type + " " +
                         "[binary] " +
                         str(msg.time))
                else:
                    send(msg.subject + " " +
                         msg.sender + " " +
                         msg.type + " " +
                         str(msg.data) + " " +
                         str(msg.time))

finally:
    cherrypy.engine.stop()

pause
#cherrypy.engine.publish("websocket-broadcast", msg)



class EchoWebSocket(WebSocket):
    def received_message(self, message):
        """
Automatically sends back the provided ``message`` to
its originating endpoint.
"""
        self.send(message.data, message.is_binary)
        # cnt = 0
        # while True:
            
        #     msg = TextMessage("hej " + str(cnt))
        #     self.send(msg.data, msg.is_binary)
        #     time.sleep(1)
        #     cnt += 1

        with Subscribe(services=[""], addr_listener=True) as sub:
            for msg in sub.recv(1):
                if msg:
                    if msg.type in ["log.debug", "log.info",
                                    "log.warning", "log.error",
                                    "log.critical"]:
                        self.send(msg.type + " " + msg.subject + " " +
                                                   msg.sender + " " +
                                                   str(msg.data) + " " +
                                                   str(msg.time))

                    elif msg.binary:
                        self.send(msg.subject + " " +
                                  msg.sender + " " +
                                  msg.type + " " +
                                  "[binary] " +
                                  str(msg.time))
                    else:
                        self.send(msg.subject + " " +
                                  msg.sender + " " +
                                  msg.type + " " +
                                  str(msg.data) + " " +
                                  str(msg.time))
#                if not self.loop:
#                    self.send("Stop logging")
#                    break

#cherrypy.quickstart(item, '/', config={'/ws': {'tools.websocket.on': True,
#                                                 'tools.websocket.handler_cls': EchoWebSocket}})

class listener( threading.Thread ):

    def __init__(self):
        threading.Thread.__init__( self )
        self.cpy = None		

    def run( self ):
        #self.cpy = cherrypy.quickstart( self )
        self.cpy = cherrypy.quickstart(item, '/', config={'/ws': {'tools.websocket.on': True,
                                                                  'tools.websocket.handler_cls': EchoWebSocket}})

    def stop( self ):
        self.cpy.stop()


interf = listener()
interf.start()

print "ready"



"""Fix the RDR files with respect to orbit number. There is a bug in the
RT-STPS software always giving the orbit number = 1"""

import os
import sys
import h5py
from datetime import datetime, timedelta
import numpy as np
import pyorbital.orbital as orb

import logging
LOG = logging.getLogger(__name__)


def fix_rdrfile(filename):
    from npp_runner.orbitno import replace_orbitno

    newname, orbnum = replace_orbitno(filename)
    os.rename(filename, newname)

    return newname, orbnum


def get_orbitnumbers_to_nppfile(npp_file):
    """Browse the RDR/SDR file and get granule start/end times, and determine
    the correct orbit numbers for those times, to be later corrected in the
    file."""
    fobj = h5py.File(npp_file, 'r')

    # First get all start and end date/time of the granules:
    obstimes = {}
    start_obstime = None
    end_obstime = None
    for group in fobj['/Data_Products'].keys():
        for dset in fobj['/Data_Products'][group]:
            # Get begining and ending date/time:
            for begin_end in ['Beginning', 'Ending']:
                try:
                    time_key = 'Aggregate' + begin_end + 'Time'
                    date_key = 'Aggregate' + begin_end + 'Date'
                    x_date = fobj[
                        '/Data_Products'][group][dset].attrs[date_key]
                    x_time = fobj[
                        '/Data_Products'][group][dset].attrs[time_key]
                    if not start_obstime and begin_end == 'Beginning':
                        # Ex.: '20120405'+'021645'
                        datetime_str = (x_date[0][0] +
                                        x_time[0][0].split('.')[0])
                        start_obstime = datetime.strptime(datetime_str,
                                                          '%Y%m%d%H%M%S')
                    if not end_obstime and begin_end == 'Ending':
                        # Ex.: '20120405'+'021645'
                        datetime_str = (x_date[0][0] +
                                        x_time[0][0].split('.')[0])
                        end_obstime = datetime.strptime(datetime_str,
                                                        '%Y%m%d%H%M%S')
                except KeyError:
                    time_key = begin_end + '_Time'
                    date_key = begin_end + '_Date'
                    x_date = fobj[
                        '/Data_Products'][group][dset].attrs[date_key]
                    x_time = fobj[
                        '/Data_Products'][group][dset].attrs[time_key]

                datetime_str = (x_date[0][0] +
                                x_time[0][0].split('.')[0])
                obstimes[group + dset + begin_end] = datetime.strptime(datetime_str,
                                                                       '%Y%m%d%H%M%S')

    fobj.close()

    if not start_obstime and not end_obstime:
        raise IOError("Failed getting the granule start and end times")

    # Check for no date (1958) problem:
    dt1958 = datetime(1958, 1, 1)
    if abs(start_obstime - dt1958) < timedelta(days=2):
        print "Start time wrong: %s" % start_obstime.strftime('%Y%m%d')
        print "Will use the end time to determine orbit number"
        if abs(end_obstime - dt1958) < timedelta(days=2):
            raise IOError("Both start time and end time is far off in file!")
        obstime = end_obstime
    else:
        obstime = start_obstime

    sat = orb.Orbital('SUOMI NPP')

    # Get the start orbit number:
    orbits = {}
    start_orbnum = sat.get_orbit_number(obstime)
    orbits['start'] = start_orbnum

    # Get all orbit numbers for the swath.
    # Check for erroneous date-times (1958):
    prev_okay_obstime = obstime
    for key in obstimes:
        if abs(obstimes[key] - dt1958) < timedelta(days=2):
            obstime = prev_okay_obstime
        else:
            obstime = obstimes[key]

        orbits[key] = sat.get_orbit_number(obstime)

    print "Orbit numbers in swath:"
    for key in obstimes:
        print orbits[key],
    print

    return orbits


def fix_nppfile4orbitnumber(orbits, npp_file, outdir):
    """Fix the NPP VIIRS RDR/SDR files with respect to oribit number. The
    RDR/SDR file is read and the correct orbit number is inserted, and the
    filename is fixed as well."""
    import shutil

    # Determine the new output filename:
    start_orbnum = orbits['start']
    npp_filename = os.path.basename(npp_file)
    new_filename = (npp_filename.split('_b')[0] +
                    '_b%.5d' % (start_orbnum) +
                    npp_filename[npp_filename.find('_c')::])
    outfile = os.path.join(outdir, new_filename)

    if os.path.exists(outfile):
        print "File exists! %s" % os.path.basename(outfile)
        return outfile

    shutil.copy(npp_file, outfile)

    # Start browsing file and change the orbit number:
    out = h5py.File(outfile)

    for group in out['/Data_Products'].keys():
        for dset in out['/Data_Products'][group]:
            # Attributes:
            subsubg_attrs = out['/Data_Products'][group][dset].attrs
            for key, val in subsubg_attrs.items():
                if key in ['AggregateBeginningOrbitNumber',
                           'N_Beginning_Orbit_Number']:
                    start_orbnum = orbits[group + dset + 'Beginning']
                    # print "Change orbit number from 1", start_orbnum
                    val = np.array([[start_orbnum]]).astype('uint64')

                    out['/Data_Products'][group][dset].attrs[key] = val

                if key in ['AggregateEndingOrbitNumber']:
                    end_orbnum = orbits[group + dset + 'Ending']
                    # print "Change orbit number from 1", end_orbnum
                    val = np.array([[end_orbnum]]).astype('uint64')

                    out['/Data_Products'][group][dset].attrs[key] = val

    out.close()
    return outfile


def get_npp_orbit_number(obstime):
    """Get the orbit number for the Suomi NPP RDR/SDR file given the
    observation start time. The orbit number"""

    sat = orb.Orbital('SUOMI NPP')

    return sat.get_orbit_number(obstime)


# -------------------------
if __name__ == "__main__":
    if len(sys.argv) < 3:
        print "Usage: %s <rdr/sdr filename> <output dir>" % sys.argv[0]
        sys.exit()
    else:
        nppfile = sys.argv[1]
        output_dir = sys.argv[2]

    orbs = get_orbitnumbers_to_nppfile(nppfile)
    fix_nppfile4orbitnumber(orbs, nppfile, output_dir)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2012, 2014 Martin Raspaud

# Author(s):

#   Kristian Rune Larsen <krl@dmi.dk>
#   Martin Raspaud <martin.raspaud@smhi.se>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Trigger and region_collector test file

AVHR_HRP_00_M01_20141017122200Z_20141017122300Z_N_O_20141017122450Z.bz2  avhrr_20141016_130300_noaa19.hrp.bz2
SVMC_npp_d20141017_t1139095_e1140339_b15396_c20141017114526433119_eum_ops.h5.bz2 AVHR_xxx_1B_M02_20141017122203Z_20141017122503Z_N_O_20141017133010Z
"""

from trollduction.collectors import trigger
from trollduction.collectors import region_collector
from pyresample import utils
import json
import os
from datetime import datetime, timedelta


input_dirs = ['tests/data', ]

regions = [
    utils.load_area('/home/a001673/usr/src/satprod/etc/areas.def', 'scan1')]

timeliness = timedelta(days=7000)
collectors = [region_collector.RegionCollector(
    region, timeliness) for region in regions]

# DONE timeout not handled
# DONE (kinda) should be able to handle both inotify, database events or
# posttroll messages
# DONE metadata should be decoded.


def terminator(metadata):
    """Dummy terminator function.
    """
    print metadata


def read_granule_metadata(filename):
    """Read the json metadata from *filename*.
    """
    with open(filename) as jfp:
        metadata = json.load(jfp)[0]

    metadata['uri'] = "file://" + os.path.abspath(filename)

    for attr in ["start_time", "end_time"]:
        try:
            metadata[attr] = datetime.strptime(metadata[attr],
                                               "%Y-%m-%dT%H:%M:%S.%f")
        except ValueError:
            metadata[attr] = datetime.strptime(metadata[attr],
                                               "%Y-%m-%dT%H:%M:%S")
    return metadata

decoder = read_granule_metadata


granule_trigger = trigger.InotifyTrigger(collectors, terminator,
                                         decoder, input_dirs)

try:
    granule_trigger.loop()
    print "Thank you for using pytroll! See you soon on pytroll.org."
except KeyboardInterrupt:
    print "Thank you for using pytroll! See you soon on pytroll.org."



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2014 Martin Raspaud

# Author(s):

#   Martin Raspaud <martin.raspaud@smhi.se>
#   Panu Lahtinen <panu.lahtinen@fmi.fi>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""This is the minion for the dungeon keeper
"""

import os
import time
import logging
import logging.config
logger = logging.getLogger(__name__)

import socket
import fcntl
import struct

# from http://stackoverflow.com/questions/11735821/python-get-localhost-ip

def get_interface_ip(ifname):
    """Get ip of interface *ifname*
    """
    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    return socket.inet_ntoa(fcntl.ioctl(sock.fileno(), 0x8915,
                                        struct.pack('256s',
                                                    ifname[:15]))[20:24])

def get_lan_ip(interface=None):
    """Get ip on lan
    """
    if interface is None:
        interfaces = [
            "eth0",
            "eth1",
            "eth2",
            "eth3",
            "eth4",
            "wlan0",
            "wlan1",
            "wifi0",
            "ath0",
            "ath1",
            "ppp0",
            ]
    else:
        interfaces = [interface]
    for ifname in interfaces:
        try:
            ip_ = get_interface_ip(ifname)
            break
        except IOError:
            pass
    try:
        return ip_
    except UnboundLocalError:
        return socket.gethostbyname(socket.gethostname())

from threading import Thread, Event
from posttroll.publisher import NoisyPublisher, context
from posttroll.message import Message

class Heart(Thread):
    """Send heartbeats once in a while.

    *pub* is the publisher to use. If it's None, a new publisher
    will be created.
    *interval* is the interval to send heartbeat on, in seconds.
    *kwargs* is the things you want to send with the beats.
    """

    def __init__(self, pub, interval=30, **kwargs):
        Thread.__init__(self)
        self._loop = True
        self._event = Event()
        self._to_send = kwargs
        self._interval = interval
        if pub is not None:
            self._pub = pub
            self._stop_pub = False
        else:
            self._pub = NoisyPublisher("Heart", 0)
            self._pub.start()
            self._stop_pub = True

    def run(self):
        while self._loop:
            msg = Message("/heart/minion", "heartbeat", self._to_send).encode()
            self._pub.send(msg)
            self._event.wait(self._interval)

    def stop(self):
        """Cardiac arrest
        """
        self._loop = False
        if self._stop_pub:
            self._pub.stop()
        self._event.set()

import zmq

class CommandReceiver(object):
    """Receive commands from an higher entity.
    """
    def __init__(self):
        self._response = None
        self._response_port = None
        self._loop = False
        self.thr = None
        self._server = None

    def start(self):
        """Start the process.
        """
        self._server = context.socket(zmq.REP)
        self._response_port = self._server.bind_to_random_port("tcp://*")
        self._loop = True
        self.thr = Thread(target=self._loop_fun)
        self.thr.start()

    def stop(self):
        """Stop the process
        """
        self._loop = False
        self._server.setsockopt(zmq.LINGER, 1)
        self._server.close()

    def restart(self):
        """Restart the process.
        """
        logger.debug("restart requested")
        self.stop()
        self.start()

    def poke(self):
        """Say if we're alive
        """
        return self._loop

    def reload(self):
        """Reload config
        """
        logger.debug("reload requested")


    def status(self):
        """Send back the status of the process (running or not)
        """
        logger.debug("status requested")

    def _loop_fun(self):
        while self._loop:
            request = self._server.recv()
            try:
                getattr(self, request)()
            except Exception as e:
                self._server.send("failed: " + str(e))
            else:
                self._server.send("done")


class Minion(CommandReceiver):
    """A simple wrapper to include a heartbeat and a command receiver.
    """
    def __init__(self):
        CommandReceiver.__init__(self)
        logger.debug("init")
        self._heart = None

    def start(self):
        CommandReceiver.start(self)
        url = "tcp://" + str(get_lan_ip()) + ":" + str(self._response_port)
        self._heart = Heart(None, interval=1,
                            pid=os.getpid(),
                            url=url)
        self._heart.start()

    def stop(self):
        """Stop minion
        """
        self._heart.stop()
        CommandReceiver.stop(self)


if __name__ == '__main__':
    logging.config.fileConfig("../etc/logging.cfg")
    logger = logging.getLogger("minion")

    try:
        minion = Minion()
        minion.start()
        while True:
            time.sleep(1000)
    except KeyboardInterrupt:
        logger.debug("Interrupted")
    except Exception as e:
        print e
        logger.exception("Something bad happened…")
    finally:
        minion.stop()
        logging.shutdown()
        time.sleep(0.5)
        context.term()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2014 Adam.Dybbroe

# Author(s):

#   Adam.Dybbroe <adam.dybbroe@smhi.se>
#   Lars Ørum Rasmussen <ras@dmi.dk>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
"""
import os
from datetime import datetime, timedelta
import re
import ConfigParser

import logging
LOG = logging.getLogger(__name__)

_RE_NPP_STAMP = re.compile(
    r'.*?(([A-Za-z0-9]+)_d(\d+)_t(\d+)_e(\d+)_b(\d+)).*')


class NPPStamp(object):

    """ A NPP stamp is:
    <platform>_d<start_date>_t<start_time>_e<end_time>_b<orbit_number>
    """

    def __init__(self, platform, start_time, end_time, orbit_number):
        self.platform = platform
        self.start_time = start_time
        self.end_time = end_time
        self.orbit_number = orbit_number

    def __str__(self):
        date = self.start_time.strftime('%Y%m%d')
        start = (self.start_time.strftime('%H%M%S') +
                 str(self.start_time.microsecond / 100000)[0])
        end = (self.end_time.strftime('%H%M%S') +
               str(self.end_time.microsecond / 100000)[0])
        return "%s_d%s_t%s_e%s_b%05d" % (self.platform, date, start, end,
                                         self.orbit_number)

    def __cmp__(self, other):
        return cmp(str(self), str(other))


def get_npp_stamp(filename):
    """A unique stamp for a granule.
    <name>_d<date>_t<start-time>_e<end-time>_b<orbit_number>
    """
    match = _RE_NPP_STAMP.match(os.path.basename(filename))
    if not match:
        return
    start_time, end_time = _dte2time(match.group(3), match.group(4),
                                     match.group(5))
    return NPPStamp(match.group(2), start_time, end_time,
                    int(match.group(6)))


def _dte2time(date, start_time, end_time):
    start_time = (datetime.strptime(date + start_time[:6], '%Y%m%d%H%M%S') +
                  timedelta(microseconds=int(start_time[6]) * 100000))
    end_time = (datetime.strptime(date + end_time[:6], '%Y%m%d%H%M%S') +
                timedelta(microseconds=int(end_time[6]) * 100000))
    if start_time > end_time:
        end_time += timedelta(days=1)
    return start_time, end_time


def get_datetime_from_filename(filename):
    """Get start observation time from the filename. Example:
    'GMODO_npp_d20120405_t0037099_e0038341_b00001_c20120405124731856767_cspp_dev.h5'
    """

    bname = os.path.basename(filename)
    sll = bname.split('_')
    return datetime.strptime(sll[2] + sll[3][:-1],
                             "d%Y%m%dt%H%M%S")



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2014

# Author(s):

#   Panu Lahtinen <pnuu+git@iki.fi>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

'''XML reader for Trollduction system and product configuration files.
'''

import xml.etree.ElementTree as etree
import os
import logging

logger = logging.getLogger(__name__)


class InfoObject(object):

    def __init__(self, **attributes):
        self.info = attributes

    def __getattr__(self, name):
        try:
            return self.info[name]
        except KeyError:
            raise AttributeError

    def get(self, key, default=None):
        return self.info.get(key, default)


class Dataset(InfoObject):

    def __init__(self, data, **attributes):
        InfoObject.__init__(self, **attributes)
        self.data = data

    def __str__(self):
        return str(self.data) + "\n" + str(self.info)

    def __repr__(self):
        return repr(self.data) + "\n" + repr(self.info)

    def copy(self, copy_data=True):
        if copy_data:
            data = self.data.copy()
        else:
            data = self.data
        return Dataset(data, **self.info)


class ProductList(object):

    def __init__(self, fname):
        self.fname = fname

        tree = etree.parse(fname)
        self._xml = tree.getroot()
        self.pl = None
        self.attrib = {}
        self.vars = {}
        self.aliases = {}
        self.groups = []
        self.parse()

    def insert_vars(self):
        """Variable replacement
        """
        for item in self.pl.getiterator():
            for key in self.vars:
                if key in item.attrib and item.attrib[key] in self.vars[key]:
                    item.set(key, self.vars[key][item.attrib[key]])

    def check_groups(self):

        # create the "rest" group
        last_group = []
        for area in self.pl:
            assigned = False
            for group in self.groups:
                if area.attrib["id"] in group.data:
                    assigned = True
                    break
            if not assigned:
                last_group.append(area.attrib["id"])
        if last_group:
            self.groups.append(Dataset(last_group, id="_rest"))

        # replace area ids with actual xml area items
        groups = []
        for group in self.groups:
            new_group = Dataset([], **group.info)
            for area_id in group.data:
                assigned = False
                for area in self.pl:
                    if area.attrib["id"] == area_id:
                        new_group.data.append(area)
                        assigned = True
                        break
                if not assigned:
                    logger.warning("Couldn't find area %s in product list",
                                   area_id)
            groups.append(new_group)
        self.groups = groups

    def parse(self):
        for item in self._xml:
            if item.tag == "product_list":
                self.pl = item
            elif item.tag == "common":
                for citem in item:
                    self.attrib[citem.tag] = citem.text
            elif item.tag == "groups":
                for group in item:
                    self.groups.append(Dataset(group.text.split(","),
                                               **group.attrib))
            elif item.tag == "variables":
                if item.attrib:
                    res = [os.environ[env] != val
                           for env, val in item.attrib.items()]
                    if any(res):
                        continue
                for var in item:
                    self.vars.setdefault(
                        var.tag, {})[var.attrib["id"]] = var.text
            elif item.tag == "aliases":
                for alias in item:
                    self.aliases.setdefault(
                        alias.tag,
                        {})[alias.attrib["src"]] = alias.attrib['dst']
        self.insert_vars()
        self.check_groups()


def get_root(fname):
    '''Read XML file and return the root tree.
    '''

    tree = etree.parse(fname)
    root = tree.getroot()

    return root


def parse_xml(tree, also_empty=False):
    '''Parse the given XML file to dictionary.
    '''
    xml_dict = {}

    # this tags will always be lists, if they are present and non-empty
    listify = ['area', 'product', 'valid_satellite', 'invalid_satellite',
               'pattern', 'file_tag', 'directory']
    children = list(tree)

    if len(children) == 0:
        try:
            xml_dict = tree.text.strip()
        except AttributeError:
            pass

    for child in children:
        new_val = parse_xml(child, also_empty=also_empty)
        if len(new_val) == 0:
            if also_empty:
                xml_dict[child.tag] = ''
                continue
            else:
                continue
        if child.tag in xml_dict:
            if not isinstance(xml_dict[child.tag], list):
                xml_dict[child.tag] = [xml_dict[child.tag]]
            xml_dict[child.tag].append(new_val)
        else:
            if len(new_val) > 0:
                if child.tag in listify:
                    xml_dict[child.tag] = [new_val]
                else:
                    xml_dict[child.tag] = new_val

    return xml_dict


def get_filepattern_config(fname=None):
    '''Retrieves the filepattern configuration file for trollstalker,
    and returns the parsed XML as a dictionary.  Optional argument
    *fname* can be used to specify the file.  If *fname* is None, the
    systemwide file is read.
    '''

    if fname is None:
        fname = os.path.realpath(__file__).split(os.path.sep)[:-2]
        fname.append('etc')
        fname.append('filepattern_config.xml')
        fname = os.path.sep.join(fname)

    return parse_xml(get_root(fname), also_empty=True)





import datetime
import json


#start_time = datetime.datetime(2012,5,15,11,21)
#start_time = datetime.datetime(2012, 5, 15, 11, 5)
start_time = datetime.datetime(2014, 10, 17, 11, 34, 39)
dt = datetime.timedelta(seconds=85.5)

npp_json_template = [{
    "create_time": "2012-05-03T15:00:50.939948",
    "description": "A SDR Granule",
    "domain": "dev",
    "end_time": "2012-04-24T13:07:10.900000",
    "orbit_number": 2542,
    "platform": "suomi npp",
    "site": "smb",
    "start_time": "2012-04-24T13:05:46.700000"}]


for step in range(30):
    granule_start_time = start_time + step * dt
    npp_json_template[0]['start_time'] = granule_start_time.isoformat()
    granule_end_time = start_time + (step + 1) * dt
    npp_json_template[0]['end_time'] = granule_end_time.isoformat()
    with open("npp_%s.json" % granule_start_time.strftime("%Y%m%d%H%M"), 'w') as jfp:
        json.dump(npp_json_template, jfp, sort_keys=True, indent=4)



import os
from datetime import datetime, timedelta
import re

from pyorbital.orbital import Orbital
from npp_runner import get_npp_stamp

import logging
LOG = logging.getLogger(__name__)


class NoTleFile(Exception):
    pass

TLE_SATNAME = {'npp': 'SUOMI NPP', }
TBUS_STYLE = False

_re_replace_orbitno = re.compile(r"_b(\d{5})")


def replace_orbitno(filename):
    stamp = get_npp_stamp(filename)

    # Correct h5 attributes
    no_date = datetime(1958, 1, 1)
    epsilon_time = timedelta(days=2)
    import h5py

    def _get_a_good_time(name, obj):
        del name
        if isinstance(obj, h5py.Dataset):
            date_key, time_key = ('Ending_Date', 'Ending_Time')
            if date_key in obj.attrs.keys():
                if not good_time_val_[0]:
                    time_val = datetime.strptime(
                        obj.attrs[date_key][0][0] +
                        obj.attrs[time_key][0][0],
                        '%Y%m%d%H%M%S.%fZ')
                    if abs(time_val - no_date) > epsilon_time:
                        good_time_val_[0] = time_val

    def _check_orbitno(name, obj):
        del name
        if isinstance(obj, h5py.Dataset):
            for date_key, time_key, orbit_key in (
                ('AggregateBeginningDate', 'AggregateBeginningTime',
                 'AggregateBeginningOrbitNumber'),
                ('AggregateEndingDate', 'AggregateEndingTime',
                 'AggregateEndingOrbitNumber'),
                ('Beginning_Date', 'Beginning_Time',
                 'N_Beginning_Orbit_Number')):
                if orbit_key in obj.attrs.keys():
                    time_val = datetime.strptime(
                        obj.attrs[date_key][0][0] +
                        obj.attrs[time_key][0][0],
                        '%Y%m%d%H%M%S.%fZ')

                    # Check for no date (1958) problem:
                    if abs(time_val - no_date) < epsilon_time:
                        LOG.info("Start time wrong: %s",
                                 time_val.strftime('%Y%m%d'))
                        LOG.info("Will use the first good end time encounter " +
                                 "in file to determine orbit number")
                        time_val = good_time_val_[0]

                    orbit_val = orbital_.get_orbit_number(time_val,
                                                          tbus_style=TBUS_STYLE)
                    obj.attrs.modify(orbit_key, [[orbit_val]])
                    counter_[0] += 1

    # Correct h5 attributes
    orbital_ = Orbital(TLE_SATNAME[stamp.platform])
    orbit = orbital_.get_orbit_number(stamp.start_time, tbus_style=TBUS_STYLE)
    LOG.info("Replacing orbit number %05d with %05d",
             stamp.orbit_number, orbit)
    fp = h5py.File(filename, 'r+')
    try:
        good_time_val_ = [None]
        fp.visititems(_get_a_good_time)
        counter_ = [0]
        fp.visititems(_check_orbitno)
        if counter_[0] == 0:
            raise IOError(
                "Failed replacing orbit number in hdf5 attributes '%s'" % filename)
        LOG.info("Replaced orbit number in %d attributes", counter_[0])
    finally:
        fp.close()

    # Correct filename
    dname, fname = os.path.split(filename)
    fname, n = _re_replace_orbitno.subn('_b%05d' % orbit, fname)
    if n != 1:
        raise IOError("Failed replacing orbit number in filename '%s'" % fname)
    return os.path.join(dname, fname), orbit

if __name__ == '__main__':
    import sys
    print replace_orbitno(sys.argv[1])



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2014 Martin Raspaud

# Author(s):

#   Martin Raspaud <martin.raspaud@smhi.se>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
"""
import sys
import time
import logging
from watchdog.observers.polling import PollingObserver
from watchdog.events import LoggingEventHandler
input_dirs = ["/data/prod/satellit/metop2/",
              "/data/prod/satellit/ears/avhrr/",
              "/data/prod/satellit/ears/viirs/",
              "/data/prod/satellit/modis/lvl1/",
              "/data/prod/satellit/ears/nwc/",
              ]

if __name__ == "__main__":
    logging.basicConfig(level=logging.INFO,
                        format='%(asctime)s - %(message)s',
                        datefmt='%Y-%m-%d %H:%M:%S')
    path = sys.argv[1] if len(sys.argv) > 1 else '.'
    event_handler = LoggingEventHandler()
    observer = PollingObserver()
    for path in input_dirs:
        observer.schedule(event_handler, path, recursive=True)
    observer.start()
    try:
        while True:
            time.sleep(1)
    except KeyboardInterrupt:
        observer.stop()
    observer.join()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2014, 2015 Adam.Dybbroe

# Author(s):

#   Adam.Dybbroe <a000680@c14526.ad.smhi.se>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""AAPP Level-1 processing on NOAA and Metop HRPT Direct Readout data. Listens
for pytroll messages from Nimbus (NOAA/Metop file dispatch) and triggers
processing on direct readout HRPT level 0 files (full swaths - no granules at
the moment)
"""

import os
import sys
LVL0_DATA_HOME = os.environ.get("LVL0_DATA_HOME", '')
DATA_ROOT_DIR = os.environ.get("DATA_ROOT_DIR", '')
PPS_DATA_HOME = os.environ.get("PPS_DATA_HOME", '')
AAPP_BIN_HOME = os.environ.get("AAPP_BIN_HOME", './')
AAPP_IN_DATDIR = os.environ.get("AAPP_IN_DATDIR", '')
WORKING_DIR = os.environ.get("WRK", './')  # AAPP Run dir

#SUPPORTED_NOAA_SATELLITES = os.environ.get("SUPPORTED_NOAA_SATELLITES", '')

NOAA_RUN_SCRIPT = "%s/smhi/AAPP_RUN_NOAA_WITH_ANA" % AAPP_BIN_HOME
METOP_RUN_SCRIPT = "%s/smhi/AAPP_RUN_METOP_SMHI" % AAPP_BIN_HOME

import ConfigParser

CONFIG_PATH = os.environ.get('AAPP_CONFIG_DIR', '')

AAPP_WORKDIR = os.environ.get("AAPP_WORKDIR", '')

CONF = ConfigParser.ConfigParser()
CONF.read(os.path.join(CONFIG_PATH, "aapp_config.cfg"))

MODE = os.getenv("SMHI_MODE")
if MODE is None:
    MODE = "offline"


OPTIONS = {}
for option, value in CONF.items(MODE, raw=True):
    OPTIONS[option] = value

#SUPPORTED_NOAA_SATELLITES = OPTIONS.get('supported_noaa_satellites')
SUPPORTED_NOAA_SATELLITES = ['NOAA-19', 'NOAA-18', 'NOAA-16', 'NOAA-15']
SUPPORTED_METOP_SATELLITES = ['Metop-B', 'Metop-A', 'Metop-C']

TLE_SATNAME = {'NOAA-19': 'NOAA 19', 'NOAA-18': 'NOAA 18',
               'NOAA-15': 'NOAA 15',
               'Metop-A': 'METOP-A', 'Metop-B': 'METOP-B',
               'Metop-C': 'METOP-C'}

METOP_NAME = {'metop01': 'Metop-B', 'metop02': 'Metop-A'}
METOP_NAME_INV = {'metopb': 'metop01', 'metopa': 'metop02'}
SATELLITE_NAME = {'NOAA-19': 'noaa19', 'NOAA-18': 'noaa18',
                  'NOAA-15': 'noaa15',
                  'Metop-A': 'metop02', 'Metop-B': 'metop01',
                  'Metop-C': 'metop03'}

# metop_sats = OPTIONS.get('supported_metop_satellites')
# SUPPORTED_METOP_SATELLITES = []
# for sat in metop_sats:
#     SUPPORTED_METOP_SATELLITES.append(METOP_NAME.get(sat, sat))

SENSOR_NAMES = ['amsu-a', 'amsu-b', 'mhs', 'avhrr/3', 'hirs/4']
SENSOR_NAME_CONVERTER = {
    'amsua': 'amsu-a', 'amsub': 'amsu-b', 'hirs': 'hirs/4',
    'mhs': 'mhs', 'avhrr': 'avhrt/3'}

METOP_NUMBER = {'b': '01', 'a': '02'}

SERVERNAME = OPTIONS['servername']

_AAPP_STAT_FILE = OPTIONS.get('aapp_stat_file', os.path.join(WORKING_DIR,
                                                             'aapp_statistics.log'))

AAPP_OUT_DIR = OPTIONS['aapp_out_dir']
METOP_IN_DIR = OPTIONS['metop_lvl0_dir']


import logging
LOG = logging.getLogger(__name__)

#: Default time format
_DEFAULT_TIME_FORMAT = '%Y-%m-%d %H:%M:%S'

#: Default log format
_DEFAULT_LOG_FORMAT = '[%(levelname)s: %(asctime)s : %(name)s] %(message)s'

_AAPP_LOG_FILE = OPTIONS.get('aapp_log_file', None)

from urlparse import urlparse
import posttroll.subscriber
from posttroll.publisher import Publish
from posttroll.message import Message
from trollduction.helper_functions import overlapping_timeinterval

import tempfile
from glob import glob
import os
import shutil
import aapp_stat
import threading
from subprocess import Popen, PIPE, STDOUT
from datetime import timedelta, datetime


def nonblock_read(output):
    """An attempt to catch any hangup in reading the output (stderr/stdout)
    from subprocess"""
    import fcntl
    fd = output.fileno()

    fl = fcntl.fcntl(fd, fcntl.F_GETFL)
    fcntl.fcntl(fd, fcntl.F_SETFL, fl | os.O_NONBLOCK)
    try:
        return output.readline()
    except:
        return ''


def reset_job_registry(objdict, key, start_end_times):
    """Remove job key from registry"""

    LOG.debug("Register: " + str(objdict))
    starttime, endtime = start_end_times
    if key in objdict:
        if objdict[key] and len(objdict[key]) > 0:
            objdict[key].remove(start_end_times)
            LOG.debug("Release/reset job-key " + str(key) + " " +
                      str(starttime) + " " + str(endtime) + " from job registry")
            LOG.debug("Register: " + str(objdict))
            return

    LOG.warning("Nothing to reset/release - " +
                "Register didn't contain any entry matching: " +
                str(key))
    return


class AappLvl1Processor(object):

    """
    Container for the Metop/NOAA level-1 processing based on AAPP

    """

    def __init__(self):

        self.lvl1_home = OPTIONS['aapp_out_dir']
        self.fullswath = True  # Always a full swath (never HRPT granules)
        self.ishmf = False
        self.working_dir = None
        self.level0_filename = None
        self.starttime = None
        self.endtime = None
        self.platform_name = "Unknown"
        self.satnum = "0"
        self.orbit = "00000"
        self.result_files = None
        self.level0files = None

        self.job_register = {}

        self.initialise()

    def initialise(self):
        """Initialise the processor"""
        self.working_dir = None
        self.level0_filename = None
        self.starttime = None
        self.endtime = None
        self.platform_name = "Unknown"
        self.satnum = "0"
        self.orbit = "00000"
        self.result_files = []
        self.level0files = {}

    def cleanup_aapp_workdir(self):
        """Clean up the AAPP working dir after processing"""

        filelist = glob('%s/*' % self.working_dir)
        dummy = [os.remove(s) for s in filelist if os.path.isfile(s)]
        filelist = glob('%s/*' % self.working_dir)
        LOG.info("Number of items left after cleaning working dir = " +
                 str(len(filelist)))
        LOG.debug("Files: " + str(filelist))
        shutil.rmtree(self.working_dir)
        return

    def pack_aapplvl1_files(self, subd):
        return pack_aapplvl1_files(self.result_files, self.lvl1_home, subd,
                                   self.satnum)

    def move_lvl1dir(self):
        if len(self.result_files) == 0:
            LOG.warning("No files in directory to move!")
            return {}

        # Get the subdirname:
        path = os.path.dirname(self.result_files[0])
        subd = os.path.basename(path)
        LOG.debug("path = " + str(path))
        LOG.debug("lvl1_home = " + self.lvl1_home)
        try:
            shutil.move(path, self.lvl1_home)
        except shutil.Error:
            LOG.warning("Directory already exists: " + str(subd))

        if self.orbit == '00000' or self.orbit == None:
            # Extract the orbit number from the sub-dir name:
            dummy, dummy, dummy, self.orbit = subd.split('_')

        # Return a dict with sensor and level for each filename:
        filenames = glob(os.path.join(self.lvl1_home, subd, '*'))
        LOG.info(filenames)

        retv = {}
        for fname in filenames:
            mstr = os.path.basename(fname).split('_')[0]
            if mstr == 'hrpt':
                lvl = '1b'
                instr = 'avhrr/3'
            else:
                lvl = mstr[-2:]
                try:
                    instr = SENSOR_NAME_CONVERTER[mstr[0:-3]]
                except KeyError:
                    LOG.warning("Sensor name will not be converted %s" %
                                str(mstr[0:-3]))
                    LOG.debug("mstr = " + str(mstr))
                    instr = mstr[0:-3]

            retv[fname] = {'level': lvl, 'sensor': instr}

        LOG.info(str(retv))

        return retv

    def run(self, msg):
        """Start the AAPP level 1 processing on either a NOAA HRPT file or a
        set of Metop HRPT files"""

        # Avoid 'collections' and other stuff:
        if msg is None or msg.type != 'file':
            return True

        LOG.debug("Received message: " + str(msg))
        LOG.debug(
            "Supported Metop satellites: " + str(SUPPORTED_METOP_SATELLITES))
        try:
            if (msg.data['platform_name'] not in SUPPORTED_NOAA_SATELLITES and
                    msg.data['platform_name'] not in SUPPORTED_METOP_SATELLITES):
                LOG.info("Not a NOAA/Metop scene. Continue...")
                return True
        except Exception, err:
            LOG.warning(str(err))
            return True

        self.platform_name = msg.data['platform_name']
        LOG.debug("Satellite = " + str(self.platform_name))

        LOG.debug("")
        LOG.debug("\tMessage:")
        LOG.debug(str(msg))
        urlobj = urlparse(msg.data['uri'])
        server = urlobj.netloc
        LOG.debug('Server = <' + str(server) + '>')
        if len(server) > 0 and server != SERVERNAME:
            LOG.warning("Server %s not the current one: %s" % (str(server),
                                                               SERVERNAME))
            return True

        LOG.info("Ok... " + str(urlobj.netloc))
        LOG.info("Sat and Sensor: " + str(msg.data['platform_name'])
                 + " " + str(msg.data['sensor']))

        self.starttime = msg.data['start_time']
        try:
            self.endtime = msg.data['end_time']
        except KeyError:
            LOG.warning(
                "No end_time in message! Guessing start_time + 14 minutes...")
            self.endtime = msg.data['start_time'] + timedelta(seconds=60 * 14)

        start_orbnum = None
        try:
            import pyorbital.orbital as orb
            sat = orb.Orbital(
                TLE_SATNAME.get(self.platform_name, self.platform_name))
            start_orbnum = sat.get_orbit_number(self.starttime)
        except ImportError:
            LOG.warning("Failed importing pyorbital, " +
                        "cannot calculate orbit number")
        except AttributeError:
            LOG.warning("Failed calculating orbit number using pyorbital")
            LOG.warning("platform name = " +
                        str(TLE_SATNAME.get(self.platform_name, self.platform_name)) + " " +
                        str(self.platform_name))

        LOG.info(
            "Orbit number determined from pyorbital = " + str(start_orbnum))
        try:
            self.orbit = int(msg.data['orbit_number'])
        except KeyError:
            LOG.warning("No orbit_number in message! Set to none...")
            self.orbit = None

        if start_orbnum and self.orbit != start_orbnum:
            LOG.warning("Correcting orbit number: Orbit now = " +
                        str(start_orbnum) + " Before = " + str(self.orbit))
            self.orbit = start_orbnum
        else:
            LOG.debug(
                'Orbit number in message determined to be okay and not changed...')

        if self.platform_name in SUPPORTED_METOP_SATELLITES:
            metop_id = SATELLITE_NAME[self.platform_name].split('metop')[1]
            self.satnum = METOP_NUMBER.get(metop_id, metop_id)
        else:
            self.satnum = SATELLITE_NAME[self.platform_name].strip('noaa')

        year = self.starttime.year
        keyname = str(self.platform_name)
        LOG.debug("Keyname = " + str(keyname))
        LOG.debug("Start: job register = " + str(self.job_register))

        # Use sat id, start and end time as the unique identifier of the scene!
        if keyname in self.job_register and len(self.job_register[keyname]) > 0:
            # Go through list of start,end time tuples and see if the current
            # scene overlaps with any:
            status = overlapping_timeinterval((self.starttime, self.endtime),
                                              self.job_register[keyname])
            if status:
                LOG.warning("Processing of scene " + keyname +
                            " " + str(status[0]) + " " + str(status[1]) +
                            " with overlapping time have been launched previously")
                LOG.info("Skip it...")
                return True
            else:
                LOG.debug(
                    "No overlap with any recently processed scenes...")

        scene_id = (str(self.platform_name) + '_' +
                    self.starttime.strftime('%Y%m%d%H%M%S') +
                    '_' + self.endtime.strftime('%Y%m%d%H%M%S'))
        LOG.debug("scene_id = " + str(scene_id))

        # Check for keys representing the same scene (slightly different
        # start/end times):
        LOG.debug("Level-0files = " + str(self.level0files))
        for key in self.level0files:
            pltrfn, startt, endt = key.split('_')
            if not self.platform_name == pltrfn:
                continue
            t1_ = datetime.strptime(startt, '%Y%m%d%H%M%S')
            t2_ = datetime.strptime(endt, '%Y%m%d%H%M%S')
            if (abs(self.starttime - t1_).seconds < 60 and
                    abs(self.endtime - t2_).seconds < 60):
                # It is the same scene!
                LOG.debug(
                    "It is the same scene, though the file times differ a bit...")
                scene_id = key
                break

        LOG.debug("scene_id = " + str(scene_id))
        if scene_id in self.level0files:
            LOG.debug("Level-0files = " + str(self.level0files[scene_id]))
        else:
            LOG.debug("No level-0files yet...")

        self.level0_filename = urlobj.path
        dummy, fname = os.path.split(self.level0_filename)

        if fname.find('.hmf') > 0:
            self.ishmf = True
        else:
            LOG.info("File is not a hmf file, " +
                     "probably a Metop file or a NOAA from DMI: " + str(fname))

        LOG.debug("Sensor = " + str(msg.data['sensor']))
        LOG.debug("type: " + str(type(msg.data['sensor'])))
        if type(msg.data['sensor']) in [str, unicode]:
            sensors = [msg.data['sensor']]
        elif type(msg.data['sensor']) is list:
            sensors = msg.data['sensor']
        else:
            sensors = []
            LOG.warning('Failed interpreting sensor(s)!')

        LOG.info("Sensor(s): " + str(sensors))
        sensor_ok = False
        for sensor in sensors:
            if sensor in SENSOR_NAMES:
                sensor_ok = True
                break
        if not sensor_ok:
            LOG.info("No required sensors....")
            return True

        if scene_id not in self.level0files:
            LOG.debug("Reset level0files: scene_id = " + str(scene_id))
            self.level0files[scene_id] = []

        for sensor in sensors:
            item = (self.level0_filename, sensor)
            if item not in self.level0files[scene_id]:
                self.level0files[scene_id].append(item)
                LOG.debug("Appending item to list: " + str(item))
            else:
                LOG.debug("item already in list: " + str(item))

        if len(self.level0files[scene_id]) < 4:
            LOG.info("Not enough sensor data available yet. " +
                     "Level-0files = " +
                     str(self.level0files[scene_id]))
            return True
        else:
            LOG.info(
                "Level 0 files ready: " + str(self.level0files[scene_id]))

        if not self.working_dir:
            try:
                self.working_dir = tempfile.mkdtemp(dir=AAPP_WORKDIR)
            except OSError:
                self.working_dir = tempfile.mkdtemp()
            finally:
                LOG.info("Create new working dir...")

        LOG.info("Working dir = " + str(self.working_dir))

        my_env = os.environ.copy()
        my_env['DYN_WRK_DIR'] = self.working_dir
        LOG.info(
            "working dir: self.working_dir = " + str(self.working_dir))
        for envkey in my_env:
            LOG.debug("ENV: " + str(envkey) + " " + str(my_env[envkey]))

        if self.platform_name in SUPPORTED_NOAA_SATELLITES:
            LOG.info("This is a NOAA scene. Start the NOAA processing!")
            LOG.info("Process the file %s" % self.level0_filename)

            cmdseq = (NOAA_RUN_SCRIPT + ' -Y ' + str(year) + ' ' +
                      self.level0_filename)
            LOG.info("Command sequence: " + str(cmdseq))
            # Run the command:
            aapplvl1_proc = Popen(cmdseq,
                                  cwd=self.working_dir,
                                  shell=True, env=my_env,
                                  stderr=PIPE, stdout=PIPE)

        elif self.platform_name in SUPPORTED_METOP_SATELLITES:
            LOG.info("This is a Metop scene. Start the METOP processing!")
            LOG.info("Process the scene %s %s" %
                     (self.platform_name, str(self.orbit)))

            sensor_filename = {}
            for (fname, instr) in self.level0files[scene_id]:
                sensor_filename[instr] = os.path.basename(fname)

            for instr in sensor_filename.keys():
                if instr not in SENSOR_NAMES:
                    LOG.error("Sensor name mismatch! name = " + str(instr))
                    return True

            cmdstr = "%s -d %s -a %s -u %s -m %s -h %s -o %s" % (METOP_RUN_SCRIPT,
                                                                 METOP_IN_DIR,
                                                                 sensor_filename[
                                                                     'avhrr/3'],
                                                                 sensor_filename[
                                                                     'amsu-a'],
                                                                 sensor_filename[
                                                                     'mhs'],
                                                                 sensor_filename[
                                                                     'hirs/4'],
                                                                 AAPP_OUT_DIR)
            LOG.info("Command sequence: " + str(cmdstr))
            # Run the command:
            aapplvl1_proc = Popen(cmdstr,
                                  cwd=self.working_dir,
                                  shell=True, env=my_env,
                                  stderr=PIPE, stdout=PIPE)

        # Taking the stderr before stdout seems to be better preventing a
        # hangup!  AD, 2013-03-07
        while True:
            line = nonblock_read(aapplvl1_proc.stderr)
            if not line:
                break
            LOG.info(line)

        while True:
            line = nonblock_read(aapplvl1_proc.stdout)
            if not line:
                break
            LOG.info(line)

        aapplvl1_proc.poll()
        dummy = aapplvl1_proc.returncode
        LOG.info("Before call to communicate:")
        out, err = aapplvl1_proc.communicate()

        lines = out.splitlines()
        for line in lines:
            LOG.info(line)

        lines = err.splitlines()
        for line in lines:
            LOG.info(line)

        LOG.info("Communicate done...")

        LOG.info(
            "Ready with AAPP level-1 processing on NOAA scene: " + str(fname))
        LOG.info(
            "working dir: self.working_dir = " + str(self.working_dir))

        # Add to job register to avoid this to be run again
        if keyname not in self.job_register.keys():
            self.job_register[keyname] = []

        self.job_register[keyname].append((self.starttime, self.endtime))
        LOG.debug("End: job register = " + str(self.job_register))

        # Block any future run on this scene for x (e.g. 10) minutes from now
        t__ = threading.Timer(
            10 * 60.0, reset_job_registry, args=(self.job_register,
                                                 str(self.platform_name),
                                                 (self.starttime, self.endtime)))
        t__.start()

        LOG.debug("After timer call: job register = " + str(self.job_register))

        self.result_files = get_aapp_lvl1_files(
            self.working_dir, msg.data['platform_name'])

        LOG.info("Output files: " + str(self.result_files))

        statresult = aapp_stat.do_noaa_and_metop(self.level0_filename,
                                                 SATELLITE_NAME.get(self.platform_name,
                                                                    self.platform_name),
                                                 self.starttime)
        if os.path.exists(_AAPP_STAT_FILE):
            fd = open(_AAPP_STAT_FILE, "r")
            lines = fd.readlines()
            fd.close()
        else:
            lines = []

        lines = lines + [statresult + '\n']
        fd = open(_AAPP_STAT_FILE, "w")
        fd.writelines(lines)
        fd.close()

        return False


def aapp_rolling_runner():
    """The AAPP runner. Listens and triggers processing on Metop/NOAA HRPT
    level 0 files dispatched from Nimbus."""

    LOG.info("*** Start the NOAA/Metop HRPT AAPP runner:")

    aapp_proc = AappLvl1Processor()

    with posttroll.subscriber.Subscribe('', ['HRPT/0', 'EPS/0'], True) as subscr:
        with Publish('aapp_runner', 0) as publisher:
            while True:
                aapp_proc.initialise()
                for msg in subscr.recv(timeout=90):
                    status = aapp_proc.run(msg)
                    if not status:
                        break  # end the loop and reinitialize!

                tobj = aapp_proc.starttime
                LOG.info("Time used in sub-dir name: " +
                         str(tobj.strftime("%Y-%m-%d %H:%M")))
                if aapp_proc.platform_name.startswith('Metop'):
                    subd = create_subdirname(tobj, aapp_proc.platform_name,
                                             aapp_proc.orbit)
                    LOG.info("Create sub-directory for level-1 files: %s" %
                             str(subd))
                    level1_files = aapp_proc.pack_aapplvl1_files(subd)
                else:
                    LOG.info("Move sub-directory with NOAA level-1 files")
                    LOG.debug(
                        "Orbit BEFORE call to move_lvl1dir: " + str(aapp_proc.orbit))
                    level1_files = aapp_proc.move_lvl1dir()
                    LOG.debug(
                        "Orbit AFTER call to move_lvl1dir: " + str(aapp_proc.orbit))

                publish_level1(publisher, level1_files,
                               aapp_proc.platform_name,
                               aapp_proc.orbit,
                               aapp_proc.starttime,
                               aapp_proc.endtime)

                if aapp_proc.working_dir:
                    LOG.info("Cleaning up directory %s" %
                             aapp_proc.working_dir)
                    aapp_proc.cleanup_aapp_workdir()

    return


def publish_level1(publisher, result_files, satellite, orbit, start_t, end_t):
    """Publish the messages that AAPP lvl1 files are ready
    """
    # Now publish:
    for key in result_files:
        resultfile = key
        LOG.debug("File: " + str(os.path.basename(resultfile)))
        filename = os.path.split(resultfile)[1]
        to_send = {}
        to_send['uri'] = ('ssh://%s/%s' % (SERVERNAME, resultfile))
        to_send['uid'] = filename
        to_send['sensor'] = result_files[key]['sensor']
        to_send['platform_name'] = satellite
        to_send['orbit_number'] = int(orbit)
        to_send['format'] = 'AAPP-HRPT'
        to_send['type'] = 'Binary'
        to_send['data_processing_level'] = result_files[key]['level']
        LOG.debug('level in message: ' + str(to_send['data_processing_level']))
        environment = MODE
        to_send['start_time'], to_send['end_time'] = start_t, end_t
        msg = Message('/' + str(to_send['format']) + '/' +
                      str(to_send['data_processing_level']) +
                      '/norrköping/' + environment + '/polar/direct_readout/',
                      "file", to_send).encode()
        LOG.debug("sending: " + str(msg))
        publisher.send(msg)


def get_aapp_lvl1_files(level1_dir, satid):
    """Get the aapp level-1 filenames for the NOAA/Metop direct readout
    swath"""

    if satid in SUPPORTED_METOP_SATELLITES:
        # Level 1b/c data:
        lvl1_files = (glob(os.path.join(level1_dir, '*.l1b')) +
                      glob(os.path.join(level1_dir, '*.l1c')) +
                      glob(os.path.join(level1_dir, '*.l1d')))
    else:
        # SUBDIR example: noaa18_20140826_1108_47748
        LOG.debug(
            'level1_dir = ' + str(level1_dir) + ' satid  = ' + str(satid))
        matchstr = os.path.join(
            level1_dir, SATELLITE_NAME.get(satid, satid) + '_????????_????_?????/') + '*'
        LOG.debug(matchstr)
        lvl1_files = glob(matchstr)

    return lvl1_files


def create_subdirname(obstime, satid, orbnum):
    """Generate the pps subdirectory name from the start observation time, ex.:
    'noaa19_20120405_0037_02270'"""
    return (SATELLITE_NAME.get(satid, satid) +
            obstime.strftime('_%Y%m%d_%H%M_') +
            '%.5d' % orbnum)


def pack_aapplvl1_files(aappfiles, base_dir, subdir, satnum):
    """Copy the AAPP lvl1 files to the sub-directory under the pps directory
    structure"""
    # aman => amsua
    # ambn => amsub (satnum <= 17)
    # ambn => mhs (satnum > 17)
    # hrsn => hirs
    # msun => msu

    # Store the sensor name and the level corresponding to the file:
    sensor_and_level = {}

    name_converter = {'avhr': 'avhrr',
                      'aman': 'amsua',
                      'hrsn': 'hirs',
                      'msun': 'msu',
                      'hrpt': 'hrpt'
                      }
    not_considered = ['dcsn', 'msun']
    path = os.path.join(base_dir, subdir)
    if not os.path.exists(path):
        os.mkdir(path)

    LOG.info("Number of AAPP lvl1 files: " + str(len(aappfiles)))
    # retvl = []
    for aapp_file in aappfiles:
        fname = os.path.basename(aapp_file)
        in_name, ext = fname.split('.')
        if in_name in not_considered:
            continue

        if in_name == 'ambn':
            instr = 'mhs'
            try:
                if int(satnum) <= 17:
                    instr = 'amsub'
            except ValueError:
                pass
            firstname = instr + ext
            level = ext.strip('l')
        elif in_name == 'hrpt':
            firstname = name_converter.get(in_name)
            instr = 'avhrr/3'
            # Could also be 'avhrr'. Will anyhow be converted below...
            level = '1b'
        else:
            instr = name_converter.get(in_name, in_name)
            LOG.debug("Sensor = " + str(instr) + " from " + str(in_name))
            firstname = instr + ext
            level = ext.strip('l')

        newfilename = os.path.join(path, "%s_%s.%s" % (firstname,
                                                       subdir, ext))
        LOG.info("Copy aapp-file to destination: " + newfilename)
        shutil.copy(aapp_file, newfilename)
        # retvl.append(newfilename)
        sensor_and_level[newfilename] = {
            'sensor': SENSOR_NAME_CONVERTER.get(instr, instr),
            'level': level}

    return sensor_and_level
    # return retvl

# ----------------------------------
if __name__ == "__main__":

    from logging import handlers

    if _AAPP_LOG_FILE:
        ndays = int(OPTIONS["log_rotation_days"])
        ncount = int(OPTIONS["log_rotation_backup"])
        handler = handlers.TimedRotatingFileHandler(_AAPP_LOG_FILE,
                                                    when='midnight',
                                                    interval=ndays,
                                                    backupCount=ncount,
                                                    encoding=None,
                                                    delay=False,
                                                    utc=True)

        handler.doRollover()
    else:
        handler = logging.StreamHandler(sys.stderr)

    handler.setLevel(logging.DEBUG)
    formatter = logging.Formatter(fmt=_DEFAULT_LOG_FORMAT,
                                  datefmt=_DEFAULT_TIME_FORMAT)
    handler.setFormatter(formatter)
    logging.getLogger('').addHandler(handler)
    logging.getLogger('').setLevel(logging.DEBUG)
    logging.getLogger('posttroll').setLevel(logging.INFO)

    LOG = logging.getLogger('aapp_runner')

    aapp_rolling_runner()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2014 Martin Raspaud

# Author(s):

#   Martin Raspaud <martin.raspaud@smhi.se>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Concatenate granules at low level if needed.
"""

from posttroll.subscriber import Subscribe
from posttroll.publisher import Publish
from posttroll.message import Message
from ConfigParser import RawConfigParser
import logging
from trollsift import compose
from datetime import datetime
import bz2
import os.path

logger = logging.getLogger(__name__)

config = RawConfigParser()


if __name__ == '__main__':

    import argparse

    parser = argparse.ArgumentParser()
    parser.add_argument("-l", "--log", help="File to log to (defaults to stdout)",
                        default=None)
    parser.add_argument("-v", "--verbose", help="print debug messages too",
                        action="store_true")
    parser.add_argument("config", help="config file to be used")
    opts = parser.parse_args()

    config.read(opts.config)

    if opts.log:
        import logging.handlers
        handler = logging.handlers.TimedRotatingFileHandler(opts.log,
                                                            "midnight",
                                                            backupCount=7)
    else:
        handler = logging.StreamHandler()
    handler.setFormatter(logging.Formatter("[%(levelname)s: %(asctime)s :"
                                           " %(name)s] %(message)s",
                                           '%Y-%m-%d %H:%M:%S'))
    if opts.verbose:
        loglevel = logging.DEBUG
    else:
        loglevel = logging.INFO
    handler.setLevel(loglevel)
    logging.getLogger('').setLevel(loglevel)
    logging.getLogger('').addHandler(handler)
    logging.getLogger("posttroll").setLevel(logging.INFO)
    logger = logging.getLogger("catter")

    subjects = list(set([config.get(section, "subject")
                         for section in config.sections()]))
    logger.info("Listening to %s", str(subjects))
    with Publish("catter") as pub:
        with Subscribe("", subjects, addr_listener=True) as sub:
            logger.info("Ready, waiting for messages")
            for msg in sub.recv():
                logger.info("Got message for %s", msg.subject)
                if msg.type == "collection":
                    mda = msg.data[0].copy()
                    section = (mda["platform"] + " " +
                               mda["number"] + "/" +
                               mda["level"])
                    if section not in config.sections():
                        logger.debug("Skipping %s", section)
                        continue
                    logger.debug("Starting catting for %s", section)
                    cat = config.get(section, "cat")
                    pattern = config.get(section, "pattern")
                    mda["proc_time"] = datetime.utcnow()
                    mda["end_time"] = msg.data[-1]["end_time"]
                    fname = compose(pattern, mda)
                    mda["uri"] = fname
                    mda["filename"] = os.path.basename(fname)
                    if cat == "bz2":
                        with open(fname, "wb") as out:
                            for cmda in msg.data:
                                infile = bz2.BZ2File(cmda["uri"], "r")
                                out.write(infile.read())
                                infile.close()
                    new_msg = Message(msg.subject, "file", mda)
                    logger.info("Done")
                    logger.debug("Sending %s", str(new_msg))
                    pub.send(str(new_msg))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2013, 2014 Adam.Dybbroe

# Author(s):

#   Adam.Dybbroe <a000680@c14526.ad.smhi.se>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Level-1 processing for VIIRS Suomi NPP Direct Readout data. Using the CSPP
level-1 processor from the SSEC, Wisconsin, based on the ADL from the NASA DRL.
Listen for pytroll messages from Nimbus (NPP file dispatch) and trigger
processing on direct readout RDR data (granules or full swaths)
"""


import os
from glob import glob
from datetime import datetime, timedelta
from urlparse import urlunsplit

import npp_runner
import npp_runner.orbitno

CSPP_SDR_HOME = os.environ.get("CSPP_SDR_HOME", '')
CSPP_RT_SDR_LUTS = os.path.join(CSPP_SDR_HOME, 'anc/cache/luts')
CSPP_WORKDIR = os.environ.get("CSPP_WORKDIR", '')
APPL_HOME = os.environ.get('NPP_SDRPROC', '')

MODE = os.getenv("SMHI_MODE")
if MODE is None:
    MODE = "dev"

VIIRS_SATELLITES = ['Suomi-NPP', 'JPSS-1', 'JPSS-2']

from urlparse import urlparse
import posttroll.subscriber
from posttroll.publisher import Publish
from posttroll.message import Message

from npp_runner import get_datetime_from_filename
from npp_runner.post_cspp import (get_sdr_files,
                                  create_subdirname,
                                  pack_sdr_files, make_okay_files,
                                  cleanup_cspp_workdir)
from npp_runner.pre_cspp import fix_rdrfile

#: Default time format
_DEFAULT_TIME_FORMAT = '%Y-%m-%d %H:%M:%S'

#: Default log format
_DEFAULT_LOG_FORMAT = '[%(levelname)s: %(asctime)s : %(name)s] %(message)s'

import os
import sys
_NPP_SDRPROC_LOG_FILE = os.environ.get('NPP_SDRPROC_LOG_FILE', None)
import logging

LOG = logging.getLogger(__name__)


# ---------------------------------------------------------------------------
def check_lut_files(thr_days=14):
    """Check if the LUT files under ${path_to_cspp_cersion}/anc/cache/luts are
    available and check if they are fresh. Return True if fresh/new files
    exists, otherwise False.
    It is files like these (with incredible user-unfriendly) names:
    510fc93d-8e4ed-6880946f-f2cdb929.asc
    510fc93d-8e4ed-6880946f-f2cdb929.VIIRS-SDR-GAIN-LUT_npp_20120217000000Z_20120220000001Z_ee00000000000000Z_PS-1-N-CCR-12-330-JPSS-DPA-002-PE-_noaa_all_all-_all.bin
    etc...

    We do not yet know if these files are always having the same name or if the
    number of files are expected to always be the same!?  Thus searching and
    checking is a bit difficult. We check if there are any files at all, and then
    how old the latest file is, and hope that this is sufficient.

    """
    import stat

    now = datetime.utcnow()

    tdelta = timedelta(
        seconds=float(URL_DOWNLOAD_TRIAL_FREQUENCY_HOURS) * 3600.)
    # Get the time of the last update trial:
    files = glob(LUT_UPDATE_STAMPFILE_RPEFIX + '*')
    # Can we count on glob sorting the most recent file first. In case we can,
    # we don't need to check the full history of time stamp files. This will
    # save time! Currently we check all files...
    # FIXME!
    update_it = True
    for filename in files:
        tslot = datetime.strptime(
            os.path.basename(filename).split('.')[-1], '%Y%m%d%H%M')
        if now - tslot < tdelta:
            update_it = False
            break

    if not update_it:
        LOG.info('An URL update trial has been attempted recently. Continue')
        return True

    LOG.info('No update trial seems to have been attempted recently')
    tdelta = timedelta(days=int(thr_days))

    files_ok = True
    LOG.info("Directory " + str(LUT_DIR) + "...")
    files = glob(os.path.join(LUT_DIR, '*'))
    if len(files) == 0:
        LOG.info("No LUT files available!")
        return False

    filename = files[0]
    tstamp = os.stat(filename)[stat.ST_MTIME]
    first_time = datetime.utcfromtimestamp(tstamp)
    #tstamp = os.stat(files[-1])[stat.ST_MTIME]
    #last_time = datetime.utcfromtimestamp(tstamp)

    if (now - first_time) > tdelta:
        LOG.info("File too old! File=%s " % filename)
        files_ok = False

    return files_ok


def update_lut_files():
    """
    Function to update the ancillary LUT files 

    These files need to be updated at least once every week, in order to
    achieve the best possible SDR processing.

    """
    import os
    import sys
    from datetime import datetime
    from subprocess import Popen, PIPE, STDOUT

    my_env = os.environ.copy()
    my_env['JPSS_REMOTE_ANC_DIR'] = URL_JPSS_REMOTE_LUT_DIR

    LOG.info("Start downloading....")
    # lftp -c "mirror --verbose --only-newer --parallel=2 $JPSS_REMOTE_ANC_DIR $CSPP_RT_SDR_LUTS"
    # cmdstr = ('lftp -c "mirror --verbose --only-newer --parallel=2 ' +
    #           URL_JPSS_REMOTE_ANC_DIR + ' ' + LUT_DIR + '"')
    cmdstr = OPTIONS['mirror_jpss_luts']
    LOG.info("Download command: " + cmdstr)

    lftp_proc = Popen(cmdstr, shell=True, env=my_env,
                      stderr=PIPE, stdout=PIPE)

    while True:
        line = lftp_proc.stdout.readline()
        if not line:
            break
        LOG.info(line)

    while True:
        errline = lftp_proc.stderr.readline()
        if not errline:
            break
        LOG.info(errline)

    lftp_proc.poll()

    now = datetime.utcnow()
    timestamp = now.strftime('%Y%m%d%H%M')
    filename = LUT_UPDATE_STAMPFILE_RPEFIX + '.' + timestamp
    try:
        fpt = open(filename, "w")
        fpt.write(timestamp)
    except OSError:
        LOG.warning('Failed to write LUT-update time-stamp file')
        return
    else:
        fpt.close()

    LOG.info("LUTs downloaded. LUT-update timestamp file = " + filename)

    return


def update_ancillary_files():
    """
    Function to update the dynamic ancillary data.

    These data files encompass Two Line Element (TLE) and Polar Wander (PW)
    files, and should preferably be updated daily. This is done automatically
    in CSPP if the viirs_sdr script is run without the '-l' option. However, to
    slightly speed up the processing and avoid hangups depending on internet
    connectivity this script can omit the automatic download (using the '-l'
    option) and thus the files need to be updated outside the script.

    """
    import os
    import sys
    from datetime import datetime
    from subprocess import Popen, PIPE, STDOUT

    my_env = os.environ.copy()
    my_env['JPSS_REMOTE_ANC_DIR'] = URL_JPSS_REMOTE_ANC_DIR

    LOG.info("Start downloading dynamic ancillary data " +
             "(TLE and Polar Wander files)....")

    cmdstr = OPTIONS['mirror_jpss_ancillary']
    LOG.info("Download command: " + cmdstr)

    mirror_proc = Popen(cmdstr, shell=True, env=my_env,
                        stderr=PIPE, stdout=PIPE)

    while True:
        line = mirror_proc.stdout.readline()
        if not line:
            break
        LOG.info(line)

    while True:
        errline = mirror_proc.stderr.readline()
        if not errline:
            break
        LOG.info(errline)

    mirror_proc.poll()

    now = datetime.utcnow()
    timestamp = now.strftime('%Y%m%d%H%M')
    filename = ANC_UPDATE_STAMPFILE_RPEFIX + '.' + timestamp
    try:
        fpt = open(filename, "w")
        fpt.write(timestamp)
    except OSError:
        LOG.warning('Failed to write ANC-update time-stamp file')
        return
    else:
        fpt.close()

    LOG.info("Ancillary data downloaded")
    LOG.info("Time stamp file = " + filename)

    return

# ---------------------------------------------------------------------------


def run_cspp(*viirs_rdr_files):
    """Run CSPP on VIIRS RDR files"""
    from subprocess import Popen, PIPE, STDOUT
    import time
    import tempfile

    viirs_sdr_call = OPTIONS['viirs_sdr_call']
    viirs_sdr_options = eval(CONF.get(MODE, 'viirs_sdr_options'))
    LOG.info("viirs_sdr_options = " + str(viirs_sdr_options))
    if not isinstance(viirs_sdr_options, list):
        LOG.warning("No options will be passed to CSPP")
        viirs_sdr_options = []

    try:
        working_dir = tempfile.mkdtemp(dir=CSPP_WORKDIR)
    except OSError:
        working_dir = tempfile.mkdtemp()

    # Run the command:
    cmdlist = [viirs_sdr_call]
    cmdlist.extend(viirs_sdr_options)
    cmdlist.extend(viirs_rdr_files)
    t0_clock = time.clock()
    t0_wall = time.time()
    LOG.info("Popen call arguments: " + str(cmdlist))
    viirs_sdr_proc = Popen(cmdlist,
                           cwd=working_dir,
                           stderr=PIPE, stdout=PIPE)
    while True:
        line = viirs_sdr_proc.stdout.readline()
        if not line:
            break
        LOG.info(line.strip('\n'))

    while True:
        errline = viirs_sdr_proc.stderr.readline()
        if not errline:
            break
        LOG.info(errline.strip('\n'))
    LOG.info("Seconds process time: " + str(time.clock() - t0_clock))
    LOG.info("Seconds wall clock time: " + str(time.time() - t0_wall))

    viirs_sdr_proc.poll()
    return working_dir


def get_sdr_times(filename):
    """Get the start and end times from the SDR file name
    """
    bname = os.path.basename(filename)
    sll = bname.split('_')
    start_time = datetime.strptime(sll[2] + sll[3][:-1],
                                   "d%Y%m%dt%H%M%S")
    end_time = datetime.strptime(sll[2] + sll[4][:-1],
                                 "d%Y%m%de%H%M%S")
    if end_time < start_time:
        end_time += timedelta(days=1)

    return start_time, end_time


def publish_sdr(publisher, result_files, orbit):
    """Publish the messages that SDR files are ready
    """
    if not result_files:
        return
    # Now publish:
    to_send = {}
    to_send["dataset"] = []
    for result_file in result_files:
        filename = os.path.basename(result_file)
        to_send['dataset'].append({'uri': urlunsplit(('ssh', SERVERNAME,
                                                      result_file, '', '')),
                                   'uid': filename})
    to_send['sensor'] = 'viirs'
    to_send['orbit_number'] = orbit
    to_send['platform_name'] = 'Suomi-NPP'
    to_send['format'] = 'SDR'
    to_send['type'] = 'HDF5'
    to_send['data_processing_level'] = '1b'
    to_send['start_time'], to_send['end_time'] = get_sdr_times(filename)

    msg = Message('/'.join(('',
                            'segment',
                            to_send['format'],
                            to_send['data_processing_level'],
                            'norrköping',
                            MODE,
                            'polar',
                            'direct_readout')),
                  "dataset", to_send).encode()
    # msg = Message('/oper/polar/direct_readout/norrkoping',
    #              "file", to_send).encode()
    LOG.debug("sending: " + str(msg))
    publisher.send(msg)


def spawn_cspp(current_granule, *glist):
    """Spawn a CSPP run on the set of RDR files given"""

    LOG.info("Start CSPP: RDR files = " + str(glist))
    working_dir = run_cspp(*glist)
    LOG.info("CSPP SDR processing finished...")
    # Assume everything has gone well!
    new_result_files = get_sdr_files(working_dir)
    if len(new_result_files) == 0:
        LOG.warning("No SDR files available. CSPP probably failed!")
        return working_dir, []

    LOG.info("current_granule = " + str(current_granule))
    LOG.info("glist = " + str(glist))
    if current_granule in glist and len(glist) == 1:
        LOG.info("Current granule is identical to the 'list of granules'" +
                 " No sdr result files will be skipped")
        return working_dir, new_result_files

    # Only bother about the "current granule" - skip the rest
    start_time = get_datetime_from_filename(current_granule)
    start_str = start_time.strftime("d%Y%m%d_t%H%M%S")
    result_files = [new_file
                    for new_file in new_result_files
                    if start_str in new_file]

    LOG.info("Number of results files = " + str(len(result_files)))
    return working_dir, result_files


class ViirsSdrProcessor(object):

    """
    Container for the VIIRS SDR processing based on CSPP

    """

    def __init__(self, ncpus):
        from multiprocessing.pool import ThreadPool
        self.pool = ThreadPool(ncpus)
        self.ncpus = ncpus

        self.orbit_number = 1  # Initialised orbit number
        self.platform_name = 'unknown'  # Ex.: Suomi-NPP
        self.fullswath = False
        self.cspp_results = []
        self.glist = []
        self.pass_start_time = None
        self.result_files = []
        self.sdr_home = OPTIONS['level1_home']

    def initialise(self):
        """Initialise the processor"""
        self.fullswath = False
        self.cspp_results = []
        self.glist = []
        self.pass_start_time = None
        self.result_files = []

    def pack_sdr_files(self, subd):
        return pack_sdr_files(self.result_files, self.sdr_home, subd)

    def run(self, msg):
        """Start the VIIRS SDR processing using CSPP on one rdr granule"""

        if msg:
            LOG.debug("Received message: " + str(msg))

        if self.glist and len(self.glist) > 0:
            LOG.debug("glist: " + str(self.glist))

        if msg is None and self.glist and len(self.glist) > 2:
            # The swath is assumed to be finished now
            LOG.debug("The swath is assumed to be finished now")
            del self.glist[0]
            keeper = self.glist[1]
            LOG.info("Start CSPP: RDR files = " + str(self.glist))
            self.cspp_results.append(self.pool.apply_async(spawn_cspp,
                                                           [keeper] + self.glist))
            LOG.debug("Inside run: Return with a False...")
            return False
        elif msg and not (msg.data['platform_name'] in VIIRS_SATELLITES and
                          msg.data['sensor'] == 'viirs'):
            LOG.info("Not a VIIRS scene. Continue...")
            return True
        elif msg is None:
            return True

        LOG.debug("")
        LOG.debug("\tMessage:")
        LOG.debug(str(msg))
        urlobj = urlparse(msg.data['uri'])
        LOG.debug("Server = " + str(urlobj.netloc))
        if urlobj.netloc != SERVERNAME:
            LOG.warning("Server %s not the current one: %s" % (str(urlobj.netloc),
                                                               SERVERNAME))
            return True
        LOG.info("Ok... " + str(urlobj.netloc))
        LOG.info("Sat and Instrument: " + str(msg.data['platform_name'])
                 + " " + str(msg.data['sensor']))

        start_time = msg.data['start_time']
        try:
            end_time = msg.data['end_time']
        except KeyError:
            LOG.warning(
                "No end_time in message! Guessing start_time + 86 seconds...")
            end_time = msg.data['start_time'] + timedelta(seconds=86)
        try:
            orbnum = int(msg.data['orbit_number'])
        except KeyError:
            LOG.warning("No orbit_number in message! Set to none...")
            orbnum = None

        rdr_filename = urlobj.path
        dummy, fname = os.path.split(rdr_filename)
        if not fname.endswith('.h5'):
            LOG.warning("Not an rdr file! Continue")
            return True

        # Check if the file exists:
        if not os.path.exists(rdr_filename):
            # raise IOError("File is reported to be dispatched " +
            #               "but is not there! File = " +
            #               rdr_filename)
            LOG.error("File is reported to be dispatched " +
                      "but is not there! File = " +
                      rdr_filename)
            return True

        # Do processing:
        LOG.info("RDR to SDR processing on npp/viirs with CSPP start!" +
                 " Start time = " + str(start_time))
        LOG.info("File = %s" % str(rdr_filename))
        # Fix orbit number in RDR file:
        LOG.info("Fix orbit number in rdr file...")
        try:
            rdr_filename, orbnum = fix_rdrfile(rdr_filename)
        except IOError:
            LOG.error('Failed to fix orbit number in RDR file = ' +
                      str(urlobj.path))
            import traceback
            traceback.print_exc(file=sys.stderr)
        except npp_runner.orbitno.NoTleFile:
            LOG.error('Failed to fix orbit number in RDR file = ' +
                      str(urlobj.path))
            LOG.error('No TLE file...')
            import traceback
            traceback.print_exc(file=sys.stderr)

        if orbnum:
            self.orbit_number = orbnum
        LOG.info("Orbit number = " + str(self.orbit_number))

        self.glist.append(rdr_filename)

        if len(self.glist) > 4:
            raise RuntimeError("Invalid number of granules to "
                               "process!!!")
        if len(self.glist) == 4:
            LOG.info("4 granules. Skip the first from the list...")
            del self.glist[0]
        if len(self.glist) == 3:
            LOG.info("3 granules. Keep the middle one...")
            keeper = self.glist[1]
        if len(self.glist) == 2:
            LOG.info("2 granules. Keep the first one...")
            keeper = self.glist[0]
        if len(self.glist) == 1:
            # Check start and end time and check if the RDR file
            # contains several granules (a full local swath):
            tdiff = end_time - start_time
            if tdiff.seconds > 4 * 60:
                LOG.info("RDR file contains 3 or more granules. " +
                         "We assume it is a full local swath!")
                keeper = self.glist[0]
                self.fullswath = True
            else:
                LOG.info("Only one granule. This is not enough for CSPP" +
                         " Continue")
                return True

        start_time = get_datetime_from_filename(keeper)
        if self.pass_start_time is None:
            self.pass_start_time = start_time

        LOG.info("Before call to spawn_cspp. Argument list = " +
                 str([keeper] + self.glist))
        self.cspp_results.append(self.pool.apply_async(spawn_cspp,
                                                       [keeper] + self.glist))
        if self.fullswath:
            LOG.info("Full swath. Break granules loop")
            return False

        return True

# ---------------------------------------------------------------------------


def npp_rolling_runner():
    """The NPP/VIIRS runner. Listens and triggers processing on RDR granules."""
    from multiprocessing import cpu_count

    LOG.info("*** Start the Suomi NPP SDR runner:")
    LOG.info("THR_LUT_FILES_AGE_DAYS = " + str(THR_LUT_FILES_AGE_DAYS))

    fresh = check_lut_files(THR_LUT_FILES_AGE_DAYS)
    if fresh:
        LOG.info("Files in the LUT dir are fresh...")
        LOG.info("...or download has been attempted recently! " +
                 "No url downloading....")
    else:
        LOG.warning("Files in the LUT dir are non existent or old. " +
                    "Start url fetch...")
        update_lut_files()

    LOG.info("Dynamic ancillary data will be updated. " +
             "Start url fetch...")
    update_ancillary_files()

    ncpus_available = cpu_count()
    LOG.info("Number of CPUs available = " + str(ncpus_available))
    ncpus = int(OPTIONS.get('ncpus', 1))
    LOG.info("Will use %d CPUs when running CSPP instances" % ncpus)
    viirs_proc = ViirsSdrProcessor(ncpus)

    with posttroll.subscriber.Subscribe('',
                                        ['RDR', ], True) as subscr:
        with Publish('viirs_dr_runner', 0) as publisher:
            while True:
                viirs_proc.initialise()
                for msg in subscr.recv(timeout=90):
                    status = viirs_proc.run(msg)
                    if not status:
                        break  # end the loop and reinitialize !

                tobj = viirs_proc.pass_start_time
                LOG.info("Time used in sub-dir name: " +
                         str(tobj.strftime("%Y-%m-%d %H:%M")))
                subd = create_subdirname(tobj, orbit=viirs_proc.orbit_number)
                LOG.info("Create sub-directory for sdr files: %s" % str(subd))

                LOG.info("Get the results from the multiptocessing pool-run")
                for res in viirs_proc.cspp_results:
                    working_dir, tmp_result_files = res.get()
                    # viirs_proc.working_dirs.append(working_dir)
                    # viirs_proc.result_files.extend(tmp_result_files)
                    viirs_proc.result_files = tmp_result_files
                    sdr_files = viirs_proc.pack_sdr_files(subd)
                    LOG.info("Cleaning up directory %s" % working_dir)
                    cleanup_cspp_workdir(working_dir)
                    publish_sdr(publisher, sdr_files,
                                viirs_proc.orbit_number)

                make_okay_files(viirs_proc.sdr_home, subd)

                LOG.info("Now that SDR processing has completed, " +
                         "check for new LUT files...")
                fresh = check_lut_files(THR_LUT_FILES_AGE_DAYS)
                if fresh:
                    LOG.info("Files in the LUT dir are fresh...")
                    LOG.info("...or download has been attempted recently! " +
                             "No url downloading....")
                else:
                    LOG.warning("Files in the LUT dir are " +
                                "non existent or old. " +
                                "Start url fetch...")
                    update_lut_files()

                LOG.info("Dynamic ancillary data will be updated. " +
                         "Start url fetch...")
                update_ancillary_files()

    return

# ---------------------------------------------------------------------------
if __name__ == "__main__":

    from logging import handlers
    import argparse
    import ConfigParser

    parser = argparse.ArgumentParser()
    parser.add_argument("-c", "--config-file",
                        required=True,
                        dest="config_file",
                        type=str,
                        default=None,
                        help="The file containing configuration parameters.")
    parser.add_argument("-l", "--log-file", dest="log",
                        type=str,
                        default=None,
                        help="The file to log to (stdout per default).")

    args = parser.parse_args()

    CONF = ConfigParser.ConfigParser()

    print "Read config from", args.config_file

    CONF.read(args.config_file)

    OPTIONS = {}
    for option, value in CONF.items(MODE, raw=True):
        OPTIONS[option] = value

    SERVERNAME = OPTIONS['servername']

    THR_LUT_FILES_AGE_DAYS = OPTIONS.get('threshold_lut_files_age_days', 14)
    URL_JPSS_REMOTE_LUT_DIR = OPTIONS['url_jpss_remote_lut_dir']
    URL_JPSS_REMOTE_ANC_DIR = OPTIONS['url_jpss_remote_anc_dir']
    LUT_DIR = OPTIONS.get('lut_dir', CSPP_RT_SDR_LUTS)
    LUT_UPDATE_STAMPFILE_RPEFIX = OPTIONS['lut_update_stampfile_prefix']
    ANC_UPDATE_STAMPFILE_RPEFIX = OPTIONS['anc_update_stampfile_prefix']
    URL_DOWNLOAD_TRIAL_FREQUENCY_HOURS = OPTIONS[
        'url_download_trial_frequency_hours']

    if args.log is not None:
        # handler = logging.FileHandler(_NPP_SDRPROC_LOG_FILE)
        ndays = int(OPTIONS.get("log_rotation_days", 1))
        ncount = int(OPTIONS.get("log_rotation_backup", 7))
        handler = handlers.TimedRotatingFileHandler(args.log,
                                                    when='midnight',
                                                    interval=ndays,
                                                    backupCount=ncount,
                                                    encoding=None,
                                                    delay=False,
                                                    utc=True)

        handler.doRollover()
    else:
        handler = logging.StreamHandler(sys.stderr)

    handler.setLevel(logging.DEBUG)
    formatter = logging.Formatter(fmt=_DEFAULT_LOG_FORMAT,
                                  datefmt=_DEFAULT_TIME_FORMAT)
    handler.setFormatter(formatter)
    logging.getLogger('').addHandler(handler)
    logging.getLogger('').setLevel(logging.DEBUG)
    logging.getLogger('posttroll').setLevel(logging.INFO)

    LOG = logging.getLogger('viirs_dr_runner')

    npp_rolling_runner()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2012, 2013, 2014 Martin Raspaud

# Author(s):

#   Martin Raspaud <martin.raspaud@smhi.se>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Start trollstalker and trollduction chains.

TODO:
 - Support chains that wait for messages, not files

"""

from ConfigParser import ConfigParser
import os
from urlparse import urlparse
import pyinotify
import fnmatch
import shutil
import logging, logging.handlers
import subprocess
import time
import glob
import sys
from trollduction import Trollduction
logger = logging.getLogger(__name__)

### Config management

def read_config(filename):
    """Read the config file called *filename*.
    """
    cp_ = ConfigParser()
    cp_.read(filename)

    res = {}

    for section in cp_.sections():
        if section == "default":
            continue
        res[section] = dict(cp_.items(section))

    return res


def reload_config(filename):
    """Rebuild chains if needed (if the configuration changed) from *filename*.
    """
    if os.path.abspath(filename) != os.path.abspath(cmd_args.config_file):
        return

    logger.debug("New config file detected! " + filename)

    new_chains = read_config(filename)

    old_glob = []

    for key, val in new_chains.iteritems():
        if key in chains:
            identical = True
            for key2, val2 in new_chains[key].iteritems():
                if ((key2 not in chains[key]) or
                    (chains[key][key2] != val2)):
                    identical = False
                    break
            if not identical:
                chains[key]["notifier"].stop()
                chains[key]["producer"].shutdown()
                chains[key] = val
                chains[key]["notifier"] = create_notifier(val)
                # create logger too!
                chains[key]["notifier"].start()
                old_glob.append(urlparse(val["origin"]).path)
                chains[key]["producer"] = Trollduction(val["config"])
                chains[key]["producer"].start()
                logger.debug("Updated " + key)
        else:
            # notifier
            chains[key] = val
            chains[key]["notifier"] = create_notifier(val)
            chains[key]["notifier"].start()
            old_glob.append(urlparse(val["origin"]).path)
            # create logger too!
            # batch producer
            chains[key]["producer"] = Trollduction(val["config"])
            chains[key]["producer"].start()
            logger.debug("Added " + key)
    for key in set(chains.keys()) - set(new_chains.keys()):
        chains[key]["notifier"].stop()
        del chains[key]
        logger.debug("Removed " + key)

    logger.debug("Reloaded config from " + filename)
    if old_glob:
        fnames = []
        for pattern in old_glob:
            fnames += glob.glob(os.path.join(pattern, "*"))
        if fnames:
            time.sleep(3)
            logger.debug("Touching old files")
            for fname in fnames:
                if os.path.exists(fname):
                    fp_ = open(fname, "ab")
                    fp_.close()
        old_glob = []

### Generic event handler

class EventHandler(pyinotify.ProcessEvent):
    """Handle events with a generic *fun* function.
    """

    def __init__(self, fun, *args, **kwargs):
        pyinotify.ProcessEvent.__init__(self, *args, **kwargs)
        self._fun = fun

    def process_IN_CLOSE_WRITE(self, event):
        """On closing after writing.
        """
        self._fun(event.pathname)

    def process_IN_CREATE(self, event):
        """On closing after linking.
        """
        try:
            if os.stat(event.pathname).st_nlink > 1:
                self._fun(event.pathname)
        except OSError:
            return

    def process_IN_MOVED_TO(self, event):
        """On closing after moving.
        """
        self._fun(event.pathname)



from trollstalker import create_notifier as cnot

def create_notifier(attrs):
    """Create a notifier from the specified configuration attributes *attrs*.
    """
    print attrs
    print "create a new trollstalker instance"
    url = urlparse(attrs["origin"])
    return cnot(attrs["file_tags"], int(attrs["publish_port"]),
                attrs["filepattern_cfg"], url.path)

def terminate(chains):
    for chain in chains.itervalues():
        chain["notifier"].stop()
        chain["producer"].shutdown()
    logger.info("Shutting down.")
    print ("Thank you for using pytroll/move_it."
           " See you soon on pytroll.org!")
    time.sleep(1)
    sys.exit(0)

if __name__ == '__main__':
    import argparse
    import signal

    parser = argparse.ArgumentParser()
    parser.add_argument("config_file", help="The configuration file to run on.")
    parser.add_argument("-l", "--log",
                        help="The file to log to. stdout otherwise.")
    cmd_args = parser.parse_args()

    log_format = "[%(asctime)s %(levelname)-8s] %(name)s: %(message)s"
    logger = logging.getLogger('')
    logger.setLevel(logging.DEBUG)

    if cmd_args.log:
        fh = logging.handlers.TimedRotatingFileHandler(
            os.path.join(cmd_args.log),
            "midnight",
            backupCount=7)
    else:
        fh = logging.StreamHandler()

    formatter = logging.Formatter(log_format)
    fh.setFormatter(formatter)

    logger.addHandler(fh)

    pyinotify.log.handlers = [fh]

    logger = logging.getLogger('move_it')
    logger.setLevel(logging.DEBUG)

    logger.info("Starting up")
    chains = {}

    mask = (pyinotify.IN_CLOSE_WRITE |
            pyinotify.IN_MOVED_TO |
            pyinotify.IN_CREATE)
    watchman = pyinotify.WatchManager()

    notifier = pyinotify.Notifier(watchman, EventHandler(reload_config))
    watchman.add_watch(os.path.dirname(cmd_args.config_file), mask)

    def chains_stop(*args):
        """Stop everything.
        """
        del args
        terminate(chains)

    signal.signal(signal.SIGTERM, chains_stop)

    try:
        reload_config(cmd_args.config_file)
        notifier.loop()
    except:
        logger.exception("Something went wrong")
    finally:
        logger.info("Shutting down")
        chains_stop()




#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2012, 2014 Martin Raspaud

# Author(s):

#   Kristian Rune Larsen <krl@dmi.dk>
#   Martin Raspaud <martin.raspaud@smhi.se>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Triggers for region_collectors
"""

from pyinotify import (ProcessEvent, Notifier, WatchManager,
                       IN_CLOSE_WRITE, IN_MOVED_TO)
import logging
from datetime import datetime
from fnmatch import fnmatch
import os.path
from posttroll.subscriber import NSSubscriber


LOG = logging.getLogger(__name__)


def total_seconds(td):
    return ((td.microseconds +
             (td.seconds + td.days * 24 * 3600) * 10 ** 6) / 10.0 ** 6)


class Trigger:

    """Abstract trigger class.
    """

    def __init__(self, collectors, terminator):
        self.collectors = collectors
        self.terminator = terminator

    def _do(self, metadata):
        """Execute the collectors and terminator.
        """
        if not metadata:
            return
        for collector in self.collectors:
            res = collector(metadata)
            if res:
                return self.terminator(res)


from threading import Thread, Event


class FileTrigger(Trigger, Thread):

    """File trigger, acting upon inotify events.
    """

    def __init__(self, collectors, terminator, decoder):
        Thread.__init__(self)
        Trigger.__init__(self, collectors, terminator)
        self.decoder = decoder
        self._running = True
        self.new_file = Event()

    def _do(self, pathname):
        mda = self.decoder(pathname)
        LOG.debug("mda: %s", str(mda))
        Trigger._do(self, mda)

    def add_file(self, pathname):
        """On arrival of a file.
        """
        self._do(pathname)
        self.new_file.set()

    def run(self):
        """The timeouts are handled here.
        """
        # The wait for new files is handled through the event mechanism of the
        # threading module:
        # - first a new file arrives, and an event is triggered
        # - then the new timeouts are computed
        # - if a timeout occurs during the wait, the wait is interrupted and
        #   the timeout is handled.
        while self._running:
            timeouts = [(collector, collector.timeout)
                        for collector in self.collectors
                        if collector.timeout is not None]

            if timeouts:
                next_timeout = min(timeouts, key=(lambda(x): x[1]))
                if next_timeout[1] and (next_timeout[1] < datetime.utcnow()):
                    LOG.warning("Timeout detected, terminating collector")
                    self.terminator(next_timeout[0].finish())
                else:
                    LOG.debug("Waiting %s seconds until timeout",
                              str(total_seconds(next_timeout[1] -
                                                datetime.utcnow())))
                    self.new_file.wait(total_seconds(next_timeout[1] -
                                                     datetime.utcnow()))
                    self.new_file.clear()
            else:
                self.new_file.wait()
                self.new_file.clear()

    def stop(self):
        """Stopping everything.
        """
        self._running = False
        self.new_file.set()


class InotifyTrigger(ProcessEvent, FileTrigger):

    """File trigger, acting upon inotify events.
    """

    def __init__(self, collectors, terminator, decoder, patterns):
        ProcessEvent.__init__(self)
        FileTrigger.__init__(self, collectors, terminator, decoder)
        self.input_dirs = []
        for pattern in patterns:
            self.input_dirs.append(os.path.dirname(pattern))
        self.patterns = patterns
        self.new_file = Event()

    def process_IN_CLOSE_WRITE(self, event):
        """On closing a file.
        """
        for pattern in self.patterns:
            if fnmatch(event.src_path, pattern):
                LOG.debug("New file detected (close write): " + event.pathname)
                self.add_file(event.pathname)

    def process_IN_MOVED_TO(self, event):
        """On moving a file into the directory.
        """
        for pattern in self.patterns:
            if fnmatch(event.src_path, pattern):
                LOG.debug("New file detected (moved to): " + event.pathname)
                self.add_file(event.pathname)

    def loop(self):
        """The main function.
        """
        self.start()
        try:
            # inotify interface
            wm_ = WatchManager()
            mask = IN_CLOSE_WRITE | IN_MOVED_TO

            # create notifier
            notifier = Notifier(wm_, self)

            # add watches
            for idir in self.input_dirs:
                wm_.add_watch(idir, mask)

            # loop forever
            notifier.loop()
        finally:
            self.stop()
            self.join()

try:
    from watchdog.events import FileSystemEventHandler
    from watchdog.observers.polling import PollingObserver
    from watchdog.observers import Observer

    class WatchDogTrigger(FileSystemEventHandler, FileTrigger):

        """File trigger, acting upon inotify events.
        """

        cases = {"PollingObserver": PollingObserver,
                 "Observer": Observer}

        def __init__(self, collectors, terminator, decoder, patterns, observer_class_name):
            FileSystemEventHandler.__init__(self)
            FileTrigger.__init__(self, collectors, terminator, decoder)
            self.input_dirs = []
            for pattern in patterns:
                self.input_dirs.append(os.path.dirname(pattern))
            self.patterns = patterns

            self.new_file = Event()
            self.observer = self.cases.get(observer_class_name, Observer)()

        def on_created(self, event):
            """On creating a file.
            """
            try:
                for pattern in self.patterns:
                    if fnmatch(event.src_path, pattern):
                        LOG.debug(
                            "New file detected (created): " + event.src_path)
                        self.add_file(event.src_path)
                        LOG.debug("Done adding")
                        return
            except:
                LOG.exception(
                    "Something wrong happened in the event processing!")

        def start(self):

            # add watches
            for idir in self.input_dirs:
                self.observer.schedule(self, idir)
            self.observer.start()

            FileTrigger.start(self)
            LOG.debug("Started polling")

        def stop(self):
            self.observer.stop()
            FileTrigger.stop(self)
            self.observer.join()
            self.join()

except ImportError:
    WatchDogTrigger = None


class MessageProcessor(Thread):

    """Process Messages
    """

    def __init__(self, services, topics):
        Thread.__init__(self)
        self.nssub = NSSubscriber(services, topics, True)
        self.sub = None
        self.loop = True

    def start(self):
        self.sub = self.nssub.start()
        Thread.start(self)

    def process_message(self, msg):
        del msg
        raise NotImplementedError("process_message is not implemented!")

    def run(self):
        try:
            for msg in self.sub.recv(2):
                if not self.loop:
                    break
                if msg is None:
                    continue
                self.process_message(msg)
        finally:
            self.stop()

    def stop(self):
        self.nssub.stop()
        self.loop = False


class PostTrollTrigger(FileTrigger):

    """Get posttroll messages.
    """

    def __init__(self, collectors, terminator, services, topics):
        self.mp = MessageProcessor(services, topics)
        self.mp.process_message = self.add_file
        FileTrigger.__init__(self, collectors, terminator, self.decode_message)

    def start(self):
        FileTrigger.start(self)
        self.mp.start()

    @staticmethod
    def decode_message(message):
        return message.data

    def stop(self):
        self.mp.stop()
        FileTrigger.stop(self)
